<?php

/** ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author    Michael Alegre
 * @copyright 2019-2023 LiteSpeed Technologies, Inc.
 * @since     2.0
 * ******************************************* */

namespace LsPanel;

use Lsc\Wp\Context\Context;
use Lsc\Wp\Context\ContextOption;
use Lsc\Wp\Context\RootPanelContextOption;
use Lsc\Wp\Logger;
use Lsc\Wp\LSCMException;
use Lsc\Wp\Panel\ControlPanel;
use Lsc\Wp\Panel\DirectAdmin;
use Lsc\Wp\PluginVersion;
use Lsc\Wp\WPDashMsgs;
use Lsc\Wp\WPInstallStorage;
use LsPanel\Extend\DAPanelLsadmContextOption;

/**
 * @since 2.0
 */
class DAEscalate
{

    /**
     * @since 2.1
     * @var   string
     */
    const BOOTSTRAP_FILE = '/usr/local/directadmin/plugins/litespeed/admin/DAEscalateBootstrap.php';

    /**
     * @since 2.0
     * @var   int
     */
    const EXIT_SUCC = 0;

    /**
     * @since 2.0
     * @var   int
     */
    const EXIT_ERROR = 1;

    /**
     * @since 2.0
     * @var   string
     */
    const LSWS_CMD_GET_LICENSE_STATUS = 'GET_LICENSE_STATUS';

    /**
     * @since 2.0
     * @var   string
     */
    const LSWS_CMD_GET_MAINMENU_INFO = 'GET_MAINMENU_INFO';

    /**
     * @since 2.0
     * @var   string
     */
    const LSWS_CMD_GET_SERIAL_NO = 'GET_SERIAL_NO';

    /**
     * @since 2.0
     * @var   string
     */
    const LSWS_CMD_RESTART_LSWS = 'RESTART_LSWS';

    /**
     * @since 2.0
     * @var   string
     */
    const LSWS_CMD_SWITCH_LICENSE = 'SWITCH_LICENSE';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_APPLY_VHCONF_CHANGES = 'applyVhConfChanges';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_CREATE_VHCONF_AND_SET_CACHE_ROOT = 'createVhConfAndSetCacheRoot';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_DA_VH_CACHE_ROOT_SEARCH = 'daVhCacheRootSearch';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_DO_ACTION = 'doAction';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_DATA_FILE_EXISTS = 'dataFileExists';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_FILE_EXISTS = 'fileExists';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_GET_CACHE_ROOT_SETTING = 'getCacheRootSetting';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_GET_CURR_VER = 'getCurrVer';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_GET_DOCROOT_MAP = 'getDocrootMap';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_GET_DATA_FILE_CONTENT = 'getDataFileContent';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_GET_LSWS_CACHE_ROOT_SETTING = 'getLswsCacheRootSetting';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_GET_REAL_PATH = 'getRealPath';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_GET_WP_DASH_MSG_DATA = 'getWpDashMsgData';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_IS_CACHE_ENABLED = 'isCacheEnabled';

    /**
     * @since 2.1
     * @var   string
     */
    const MGR_CMD_POPULATE_MIN_API_VER_FILE = 'populateMinApiVerFile';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_PROCESS_TEMP_ESCALATION_LOG = 'processTempEscalationLog';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_REFRESH_VER_LIST = 'refreshVerList';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_REMOVE_FILE = 'removeFile';

    /**
     * @since 2.3
     * @var   string
     */
    const MGR_CMD_SCAN_2_BATCH = 'scan2Batch';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_SET_ACTIVE_VER = 'setActiveVer';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_SET_SERVER_CACHE_ROOT = 'setServerCacheRoot';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_UPDATE_DATA_FILE = 'updateDataFile';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_UPDATE_SHARED_CODE = 'updateSharedCode';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_WRITE_DASH_PLUGIN_MD5 = 'writeDashMd5';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_WRITE_TO_LOG = 'WRITE_TO_LOG';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_WRITE_VH_CACHE_ROOT = 'writeVhCacheRoot';

    /**
     * @since 2.0
     * @var   string
     */
    const MGR_CMD_WRITE_WP_DASH_MSGS_DATA = 'writeWpDashMsgsData';

    /**
     * @since 2.0
     * @var   string
     */
    const TYPE_CMD_LSWS = 'typeCmdLsws';

    /**
     * @since 2.0
     * @var   string
     */
    const TYPE_CMD_MGR = 'typeCmdMgr';

    /**
     * @since 2.0
     * @var   string
     */
    const VAL_LSWS_HOME_NOT_SET = 'NOT_SET';

    /**
     * @since 2.0
     * @var   string
     */
    private $parentPid;

    /**
     * @since 2.0
     * @var   string
     */
    private $action;

    /**
     * @since 2.0
     * @var   array
     */
    private $params = array();

    /**
     *
     * @since 2.0
     *
     * @param string[] $input
     *
     * @throws DAPluginException  Thrown indirectly by $this->init() call.
     */
    private function __construct( array $input )
    {
        $this->init($input);
    }

    /**
     *
     * @since 2.0
     *
     * @param string[] $input
     *
     * @throws DAPluginException  Thrown when "LSWS home" value parsed from
     *     $input is invalid.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->setActionAndParams() call.
     */
    private function init( array $input )
    {
        $lswsHome = array_shift($input);

        if ( $lswsHome != self::VAL_LSWS_HOME_NOT_SET
                && ! DALiteSpeedUtil::setLswsHome($lswsHome) ) {

            throw new DAPluginException(
                'Invalid input for lswsHome.',
                self::EXIT_ERROR
            );
        }

        $this->parentPid = array_shift($input);

        $this->setActionAndParams($input);
    }

    /**
     *
     * @since 2.0
     *
     * @param string[] $cmdInput
     *
     * @throws DAPluginException  Thrown when $action value parsed from
     *     $cmdInput is unexpected.
     */
    private function setActionAndParams( array $cmdInput )
    {
        $action = array_shift($cmdInput);

        if ( self::getSupportedCmdType($action) == '' ) {
            throw new DAPluginException(
                "[ERROR] Illegal input: $action is not a valid command.\n",
                self::EXIT_ERROR
            );
        }

        $this->action = $action;

        if ( !empty($cmdInput) ) {
            $this->params = unserialize(array_shift($cmdInput));
        }
    }

    /**
     *
     * @since 2.0
     *
     * @return DirectAdmin
     *
     * @throws LSCMException  Thrown indirectly by
     *     ControlPanel::getClassInstance() call.
     */
    private function getControlPanelInstance()
    {
        /* @var DirectAdmin $panel */
        $panel = ControlPanel::getClassInstance('\Lsc\Wp\Panel\DirectAdmin');

        return $panel;
    }

    /**
     *
     * @since 2.1
     *
     * @param string[] $params
     *
     * @return string
     */
    private function checkForMissingParams( array $params )
    {
        foreach( $params as $param ) {

            if ( !isset($this->params[$param]) ) {
                return $param;
            }
        }

        return '';
    }

    /**
     *
     * @since 2.1
     *
     * @param string $functionName
     * @param string $param
     *
     * @return string
     */
    private function getMissingParamMsg( $functionName, $param )
    {
        return "Missing '$param' arg in escalated $functionName() call.";
    }

    /**
     *
     * @since 2.0
     */
    private function e2_restartLsws()
    {
        exec('/sbin/service lshttpd restart');
    }

    /**
     *
     * @since 2.0
     */
    private function e1_echoCurrentSerialNo()
    {
        $serial = 'NO SERIAL FOUND';

        if ( ($lswsHome = DALiteSpeedUtil::getLswsHome()) != '' ) {
            $serial_file = "$lswsHome/conf/serial.no";

            if ( is_file($serial_file) ) {
                $serial = file_get_contents($serial_file);
            }
            elseif ( is_file("$lswsHome/conf/trial.key") ) {
                $serial = 'TRIAL';
            }
        }

        echo "RESULT serial=$serial\n";
    }

    /**
     *
     * @since 2.0
     */
    private function e1_echoAdminUrl()
    {
        $protocol = 'http';
        $port     = "7080";

        if ( ($lswsHome = DALiteSpeedUtil::getLswsHome()) != '' ) {
            $data = file_get_contents("$lswsHome/admin/conf/admin_config.xml");

            if ( $data != '' ) {

                $isSecure = (
                    preg_match('/<secure>(\d)<\/secure>/', $data, $matches1)
                    &&
                    $matches1[1] == 1
                );

                if ( $isSecure ) {
                    $protocol .= 's';
                }

                $portFound = (bool)preg_match(
                    '/<address>.*:(\d+)<\/address>/',
                    $data,
                    $matches2
                );

                if ( $portFound ) {
                    $port = $matches2[1];
                }
            }
        }

        echo "RESULT adminurl=$protocol://{$_SERVER['SERVER_NAME']}:$port/\n";
    }

    /**
     *
     * @since 2.0
     */
    private function e1_echoOpenAdminUrl()
    {
        $protocol = 'http';
        $port     = "7080";

        if ( ($lswsHome = DALiteSpeedUtil::getLswsHome()) != '' ) {
            $data = file_get_contents("$lswsHome/admin/conf/admin_config.conf");

            if ( $data != '' ) {
                $isSecure = (
                    preg_match(
                        '/adminListener {(?:.|\n)*\ssecure\s+(\d)\s/',
                        $data,
                        $matches1
                    )
                    && $matches1[1] == 1
                );

                if ( $isSecure ) {
                    $protocol .= 's';
                }

                $portFound = (bool)preg_match(
                    '/adminListener {(?:.|\n)*\saddress\s+\*:(\d+)\s/',
                    $data,
                    $matches2
                );

                if ( $portFound ) {
                    $port = $matches2[1];
                }
            }
        }

        echo "RESULT adminurl=$protocol://{$_SERVER['SERVER_NAME']}:$port/\n";
    }

    /**
     *
     * @since 2.0
     */
    private function e1_echoMainMenuInfo()
    {
        if ( isset($this->params['serv_type']) ) {
            $servType = $this->params['serv_type'];
        }
        else {
            $servType = 'ent';
        }

        switch ( $servType ) {

            case 'ent':
                $this->e1_echoCurrentSerialNo();
                $this->e1_echoAdminUrl();
                break;

            case 'open':
                $this->e1_echoOpenAdminUrl();
                break;
        }
    }

    /**
     *
     * @since 2.0
     */
    private function e2_echoCurrentLicenseStatus()
    {
        $lswsHome    = DALiteSpeedUtil::getLswsHome();
        $licenseFile = "$lswsHome/conf/license.key";
        $buf         = '';

        if ( file_exists($licenseFile) ) {
            $result1 = $this->e2_runLicenseCheck();

            $return_var = $result1['return_var'];
            $buf       .= DALiteSpeedUtil::getOutputString($result1['output']);
        }
        else {

            if ( file_exists("$lswsHome/conf/serial.no") ) {
                $output = array();
                exec("$lswsHome/bin/lshttpd -r 2>&1", $output);

                $buf .= DALiteSpeedUtil::getOutputString($output);
            }

            if ( file_exists($licenseFile)
                    || file_exists("$lswsHome/conf/trial.key") ) {

                $result2 = $this->e2_runLicenseCheck();

                $return_var = $result2['return_var'];
                $buf .= DALiteSpeedUtil::getOutputString($result2['output']);
            }
            else {
                $return_var = -1;
                $buf       .=
                    "Failed to find serial.no or trial key file, abort!";
            }
        }

        if ( $buf != '' ) {
            echo "RESULT status=$buf\n";
        }

        echo "RESULT return_var=$return_var\n";
    }

    /**
     *
     * @since 2.0
     *
     * @return array[]
     */
    private function e2_runLicenseCheck()
    {
        exec(
            DALiteSpeedUtil::getLswsHome() . '/bin/lshttpd -V 2>&1',
            $output,
            $return_var
        );

        return array( 'return_var' => $return_var, 'output' => $output );
    }

    /**
     *
     * @since 2.0
     *
     * @return void
     *
     * @throws DAPluginException  Thrown when getmypid() call returns boolean
     *     false.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->handleSwitchLicenseError() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->handleSwitchLicenseError() call.
     */
    private function e2_switchLicense()
    {
        $error = $serial_no = '';

        if ( isset($this->params['serial_no']) ) {
            $serial_no = $this->params['serial_no'];
        }

        if ( $serial_no == 'TRIAL' ) {
            $trialKeyFound =
                file_exists(DALiteSpeedUtil::getLswsHome() . '/conf/trial.key');

            if ( !$trialKeyFound ) {
                $error = 'Failed to find trial.key file. Aborted. <br /> '
                    . 'Please request a trial license and save it under '
                    . 'lsws/conf.';
            }
        }
        else {
            $error = DALiteSpeedUtil::validate_serial_no($serial_no);
        }

        if ( $error != '' ) {
            $this->handleSwitchLicenseError($error);
            return;
        }

        if ( ($pid = getmypid()) === false ) {
            throw new DAPluginException(
                'Failed to get current PHP process ID when attempting to '
                    . 'switch license.'
            );
        }

        $renamed = array();

        $error = $this->e2_attemptSwitchLicense($serial_no, $pid, $renamed);

        if ( $error != '' ) {
            $this->handleSwitchLicenseError($error, $pid, $renamed);
            return;
        }

        if ( DALiteSpeedUtil::getLSPID() != 0 ) {
            $this->e2_restartLsws();
        }

        $this->echoSwitchLicenseResult();
    }

    /**
     *
     * @since 2.0
     *
     * @param string   $serial
     * @param int      $pid
     * @param string[] $renamed
     *
     * @return string
     */
    private function e2_attemptSwitchLicense( $serial, $pid, array &$renamed )
    {
        $lswsHome = DALiteSpeedUtil::getLswsHome();

        $serial_file  = "$lswsHome/conf/serial.no";
        $license_file = "$lswsHome/conf/license.key";

        if ( file_exists($license_file) ) {

            if ( !DALiteSpeedUtil::pidBackup($license_file, $pid) ) {
                return 'Failed to back up old license.key';
            }

            $renamed[] = $license_file;
        }

        if ( file_exists($serial_file) ) {

            if ( !DALiteSpeedUtil::pidBackup($serial_file, $pid) ) {
                return 'Failed to back up old serial.no';
            }

            $renamed[] = $serial_file;
        }

        if ( $serial == 'TRIAL' ) {
            exec("$lswsHome/bin/lshttpd -t 2>&1", $output1, $return_var1);

            if ( $return_var1 != 0 ) {
                return 'Test license failed.<br />'
                    . DALiteSpeedUtil::getOutputString($output1);
            }

            $cmd = "$lswsHome/bin/lshttpd -V 2>&1";
        }
        else {
            file_put_contents($serial_file, "$serial\n");

            $cmd = "$lswsHome/bin/lshttpd -r 2>&1";
        }

        exec($cmd, $output2, $return_var2);

        if ( $return_var2 != 0 ) {
            return 'Check license failed.<br />'
                . DALiteSpeedUtil::getOutputString($output2);
        }

        return '';
    }

    /**
     *
     * @since 2.0
     *
     * @param string   $error
     * @param int|null $pid      Optional when parameter array $renamed is
     *     empty.
     * @param string[] $renamed  Optional.
     *
     * @throws DAPluginException  Thrown when $pid parameter value is not an
     *     integer while $renamed array parameter is not empty.
     */
    private function handleSwitchLicenseError(
              $error,
              $pid = null,
        array $renamed = array() )
    {
        if ( !empty($renamed) && !is_int($pid) ) {
            throw new DAPluginException(
                'Function handleSwitchLicenseError() cannot be called with '
                    . 'non-integer $pid parameter value when $renamed array '
                    . 'parameter is not empty.'
            );
        }

        foreach ( $renamed as $file ) {
            DALiteSpeedUtil::undoPidBackup($pid, $file);
        }

        $this->echoSwitchLicenseResult($error);
    }

    /**
     *
     * @since 2.0
     *
     * @param string $error
     */
    private function echoSwitchLicenseResult( $error = '' )
    {
        if ( $error == '' ) {
            $retVar = 0;
            $statusMsg = '<p><b>Successfully switched to new license.</b></p>';
        }
        else {
            $retVar = -1;
            $statusMsg = '<p><b>Failed to switch current license.</b></p>'
                . "<p>$error</p>";
        }

        echo "RESULT status=$statusMsg\n";
        echo "RESULT return_var=$retVar\n";
    }

    /**
     *
     * @since 2.0
     *
     * @return bool
     */
    private function e2_writeToLog()
    {
        $tempLogFile = DAPluginLogger::TEMP_LOG_BASE . "_$this->parentPid.log";

        if ( file_exists($tempLogFile) ) {

            if ( ($lswsHome = DALiteSpeedUtil::getLswsHome()) != '' ) {
                /**
                 * Check that shared code ContextOption.php has been loaded
                 * (or load it here) before using it if possible. (Optional)
                 */
                $sharedContextOption = "$lswsHome/add-ons/webcachemgr/src"
                    . '/Context/ContextOption.php';
                $logDir              = "$lswsHome/logs";

                if ( file_exists($sharedContextOption)
                        && include_once $sharedContextOption ) {

                    $logFile = "$logDir/" . ContextOption::LOG_FILE_NAME;
                }
                else {
                    $logFile = "$logDir/webcachemgr.log";
                }
            }

            if ( !isset($logFile) || !file_exists(dirname($logFile)) ) {
                $logFile = '/tmp/ls-da.log';
            }

            return (bool)file_put_contents(
                $logFile,
                file_get_contents($tempLogFile),
                FILE_APPEND | LOCK_EX
            );
        }

        return false;
    }

    /**
     *
     * @since 2.3
     *
     * @return string[][]
     *
     * @throws DAPluginException  Thrown when an expected parameter is
     *     missing.
     * @throws LSCMException  Thrown indirectly by Context::getActionTimeout()
     *     call.
     * @throws LSCMException  Thrown indirectly by WPInstallStorage::scan2()
     *     call.
     */
    private function e2_scan2Batch()
    {
        $missingParam = $this->checkForMissingParams(array('docroots'));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        $docroots = $this->params['docroots'];

        $endTime = count($docroots) > 1 ? Context::getActionTimeout() : 0;

        $finishedList = $wpPaths = array();

        foreach ( $docroots as $docroot ) {
            $wpPaths = array_merge($wpPaths, WPInstallStorage::scan2($docroot));

            $finishedList[] = $docroot;

            if ( $endTime && time() >= $endTime ) {
                break;
            }
        }

        return array(
            'finishedList' => $finishedList,
            'wpPaths'      => $wpPaths
        );
    }

    /**
     *
     * @since 2.0
     *
     * @throws LSCMException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     * @throws LSCMException  Thrown indirectly by $panel->setServerCacheRoot()
     *     call.
     */
    private function e2_setServerCacheRoot()
    {
        $panel = $this->getControlPanelInstance();
        $panel->setServerCacheRoot();
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     * @throws LSCMException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     * @throws LSCMException  Thrown indirectly by $panel->writeVHCacheRoot()
     *     call.
     */
    private function e2_writeVHCacheRoot()
    {
        $missingParam = $this->checkForMissingParams(array( 'vhConf' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        $panel = $this->getControlPanelInstance();
        $panel->writeVHCacheRoot($this->params['vhConf']);
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     * @throws LSCMException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     */
    private function e2_daVhCacheRootSearch()
    {
        $missingParam = $this->checkForMissingParams(array( 'confDir' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        $panel = $this->getControlPanelInstance();

        return $panel->daVhCacheRootSearch($this->params['confDir']);
    }

    /**
     *
     * @since 2.0
     * @since 2.1  Removed optional $confFile parameter.
     *
     * @return string
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     * @throws LSCMException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     */
    private function e2_getCacheRootSetting()
    {
        $missingParam = $this->checkForMissingParams(array( 'confFile' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        $panel = $this->getControlPanelInstance();

        return $panel->getCacheRootSetting($this->params['confFile']);
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     *
     * @throws LSCMException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     */
    private function e2_getLSWSCacheRootSetting()
    {
        $panel = $this->getControlPanelInstance();

        return $panel->getLSWSCacheRootSetting();
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     * @throws LSCMException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     * @throws LSCMException  Thrown indirectly by
     *     $panel->createVHConfAndSetCacheRoot() call.
     */
    private function e2_createVHConfAndSetCacheRoot()
    {
        $missingParam = $this->checkForMissingParams(array( 'vhConf' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        $panel = $this->getControlPanelInstance();

        $panel->createVHConfAndSetCacheRoot($this->params['vhConf']);
    }

    /**
     *
     * @since 2.0
     *
     * @throws LSCMException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     * @throws LSCMException  Thrown indirectly by $panel->applyVHConfChanges()
     *     call.
     */
    private function e2_applyVHConfChanges()
    {
        $panel = $this->getControlPanelInstance();

        $panel->applyVHConfChanges();
    }

    /**
     *
     * @since 2.0
     *
     * @return array[]
     *
     * @throws LSCMException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     */
    private function e2_getDocrootMap()
    {
        $panelEnv = $this->getControlPanelInstance();

        return $panelEnv->getDocrootMap();
    }

    /**
     *
     * @since 2.0
     *
     * @return bool
     *
     * @throws LSCMException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     * @throws LSCMException  Thrown indirectly by $panel->isCacheEnabled()
     *     call.
     */
    private function e2_isCacheEnabled()
    {
        $panel = $this->getControlPanelInstance();

        return $panel->isCacheEnabled();
    }

    /**
     *
     * @since 2.0
     *
     * @return array[]
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     * @throws LSCMException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     * @throws LSCMException  Thrown indirectly by Context::getLSCMDataFiles()
     *     call.
     * @throws LSCMException  Thrown indirectly by "new WPInstallStorage()"
     *     call.
     * @throws LSCMException  Thrown indirectly by $wpInstallStorage->doAction()
     *     call.
     * @throws LSCMException  Thrown indirectly by Logger::getUiMsgs() call.
     */
    private function e2_doAction()
    {
        $missingParam = $this->checkForMissingParams(array( 'action', 'list' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        /**
         * Init DirectAdmin control panel object.
         */
        $this->getControlPanelInstance();

        $dataFileInfo     = Context::getLSCMDataFiles();
        $wpInstallStorage = new WPInstallStorage(
            $dataFileInfo['dataFile'],
            $dataFileInfo['custDataFile']
        );

        if ( isset($this->params['extraArgs']) ) {
            $extraArgs = $this->params['extraArgs'];
        }
        else {
            $extraArgs = array();
        }

        return array(
            'finishedList'    =>
                $wpInstallStorage->doAction(
                    $this->params['action'],
                    $this->params['list'],
                    $extraArgs
                ),
            'workingQueue'    => $wpInstallStorage->getWorkingQueue(),
            'invalidPathErrs' => Logger::getUiMsgs(Logger::UI_ERR)
        );
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     *
     * @throws LSCMException  Thrown indirectly by
     *     PluginVersion::getCurrentVersion() call.
     */
    private function e2_getCurrentVersion()
    {
        return PluginVersion::getCurrentVersion();
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     * @throws LSCMException  Thrown indirectly by PluginVersion::getInstance()
     *     call.
     * @throws LSCMException  Thrown indirectly by $vermgr->setActiveVersion()
     *     call.
     */
    private function e2_setActiveVersion()
    {
        $missingParam = $this->checkForMissingParams(array( 'ver' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        $vermgr = PluginVersion::getInstance();

        $vermgr->setActiveVersion(
            $this->params['ver'],
            (isset($this->params['init'])) ? $this->params['init'] : false
        );
    }

    /**
     *
     * @since 2.0
     *
     * @throws LSCMException  Thrown indirectly by PluginVersion::getInstance()
     *     call.
     */
    private function e2_refreshVersionList()
    {
        /**
         * Trigger refresh version list during init().
         */
        PluginVersion::getInstance();
    }

    /**
     *
     * @since 2.0
     *
     * @return bool
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     */
    private function e2_fileExists()
    {
        $missingParam = $this->checkForMissingParams(array( 'filePath' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        $filePath = $this->params['filePath'];

        $isApprovedFilePath = in_array(
            $filePath,
            array('/etc/httpd/conf/extra/httpd-includes.conf')
        );

        if ( !$isApprovedFilePath ) {
            return false;
        }

        return file_exists($filePath);
    }

    /**
     *
     * @since 2.0
     *
     * @return bool
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     * @throws LSCMException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     * @throws LSCMException  Thrown indirectly by Context::getLSCMDataFiles()
     *     call.
     */
    private function e2_dataFileExists()
    {
        $missingParam = $this->checkForMissingParams(array( 'filePath' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        $filePath = $this->params['filePath'];

        /**
         * Init DirectAdmin control panel object.
         */
        $this->getControlPanelInstance();

        $dataFileInfo = Context::getLSCMDataFiles();

        $isApprovedFilePath = in_array(
            $filePath,
            array(
                $dataFileInfo['dataFile'],
                $dataFileInfo['custDataFile']
            )
        );

        if ( !$isApprovedFilePath ) {
            return false;
        }

        return file_exists($filePath);
    }

    /**
     * Attempts to get the file contents of a data file based on a pre-approved
     * list.
     *
     * @since 2.0
     *
     * @return string
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     * @throws LSCMException  Thrown indirectly by Context::getLSCMDataFiles()
     *     call.
     */
    private function e2_getDataFileContent()
    {
        $missingParam = $this->checkForMissingParams(array( 'filePath' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        $filePath = $this->params['filePath'];

        $dataFilePaths = Context::getLSCMDataFiles();

        $isApprovedFilePath = in_array(
            $filePath,
            array(
                $dataFilePaths['dataFile'],
                $dataFilePaths['custDataFile']
            )
        );

        if ( !$isApprovedFilePath || !file_exists($filePath) ) {
            return '';
        }

        return file_get_contents($filePath);
    }

    /**
     * Works similar to PHP's realpath() with the exception of returning false
     * when supplied with an empty string as a path.
     *
     * @since 2.0
     *
     * @return bool|string
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     */
    private function e2_realPath()
    {
        $missingParam = $this->checkForMissingParams(array( 'filePath' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        $filePath = $this->params['filePath'];

        if ( $filePath == '' ) {
            return false;
        }

        return realpath($filePath);
    }

    /**
     *
     * @since 2.0
     *
     * @return bool
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     */
    private function e2_removeFile()
    {
        $missingParam = $this->checkForMissingParams(array( 'id' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        $files = array (
            'dashPlugin' => '/usr/src/litespeed-wp-plugin/dash-notifier'
        );

        $id = $this->params['id'];

        if ( isset($files[$id]) ) {
            $file = $files[$id];

            if ( file_exists($file) ) {

                if ( is_dir($file) ) {
                    return DALiteSpeedUtil::e2_rrmdir($file);
                }
                else {
                    return unlink($file);
                }
            }
        }

        return false;
    }

    /**
     *
     * @since 2.0
     *
     * @return int|false
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     */
    private function e2_writeWpDashMsgsData()
    {
        $missingParam = $this->checkForMissingParams(array( 'msgsJson' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        return file_put_contents(
            DALiteSpeedUtil::getLswsHome() . '/admin/lscdata/wpDashMsgs.data',
            base64_decode($this->params['msgsJson'])
        );
    }

    /**
     *
     * @since 2.0
     *
     * @return int|false
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     */
    private function e2_writeDashPluginMd5()
    {
        $missingParam = $this->checkForMissingParams(array( 'dashMd5' ));

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        return file_put_contents(
            '/usr/src/litespeed-wp-plugin/dash_md5',
            $this->params['dashMd5']
        );
    }

    /**
     *
     * @since 2.0
     *
     * @return string[]|string[][]
     */
    private function e2_getWpDashMsgData()
    {
        $wpDashMsgs = new WPDashMsgs();

        return $wpDashMsgs->getMsgData();
    }

    /**
     * Get temp escalation log file content and write it to the general
     * plugin temp log. This is used to keep logged messages in the correct
     * order chronologically.
     *
     * @since 2.0
     *
     * @return bool
     *
     * @throws DAPluginException  Thrown indirectly by
     *     DAPluginEscalatedLogger::getLogFilePath() call.
     */
    private function e2_processTempEscalationLog()
    {
        $tempEscalationLog = DAPluginEscalatedLogger::getLogFilePath();

        if ( file_exists($tempEscalationLog) ) {
            $ret = (bool)file_put_contents(
                DAPluginLogger::TEMP_LOG_BASE . "_$this->parentPid.log",
                file_get_contents($tempEscalationLog),
                FILE_APPEND | LOCK_EX
            );

            if ( $ret ) {
                unlink($tempEscalationLog);
            }

            return $ret;
        }

        return false;
    }

    /**
     *
     * @since 2.3
     *
     * @param string|string[] $message
     *
     * @return string|string[]
     */
    private static function stripLscmctlErrorMsgColorChars( $message )
    {
        return str_replace([ "\e[33m", "\e[0m" ], '', $message);
    }

    /**
     *
     * @since 2.0
     * @since 2.3  Return type changed from 'bool' to 'string'.
     *
     * @return string
     */
    private function e2_updateSharedCode()
    {
        $output = array();

        DALiteSpeedUtil::exec_cmd(
            DALiteSpeedUtil::getLswsHome()
                . '/admin/misc/lscmctl --force-update-lib 2>&1 >/dev/null',
            $output,
            $ret
        );

        if ( $ret == 0 ) {
            return '';
        }

        if ( !empty($output) ) {

            foreach ( $output as $line ) {

                if ( strpos($line, "ERROR:") === 5 ) {
                    return self::stripLscmctlErrorMsgColorChars($line);
                }
            }
        }

        return '"lscmctl --update-lib" command failed.';
    }

    /**
     *
     * @since 2.0
     *
     * @return bool
     *
     * @throws DAPluginException  Thrown when an expected parameter is missing.
     * @throws LSCMException  Thrown indirectly by
     *     WPInstallStorage::updateDataFile() call.
     */
    private function e2_updateDataFile()
    {
        $missingParam = $this->checkForMissingParams(
            array( 'dataFile', 'dataFileVer' )
        );

        if ( $missingParam != '' ) {
            throw new DAPluginException(
                $this->getMissingParamMsg(__FUNCTION__, $missingParam)
            );
        }

        return WPInstallStorage::updateDataFile(
            $this->params['dataFile'],
            $this->params['dataFileVer']
        );
    }

    /**
     *
     * @since 2.1
     */
    private function e2_populateMinAPIVerFile()
    {
        ControlPanel::populateMinAPIVerFile();
    }

    /**
     *
     * @since 2.0
     * @since 2.2  Added parameter $pid and placed it first in the function
     *     signature.
     *
     * @param int    $pid
     * @param bool   $asRoot
     * @param string $cmd
     * @param array  $args
     *
     * @return string
     */
    private static function getEscalateCmd(
              $pid,
              $asRoot,
              $cmd,
        array $args = array() )
    {
        if ( ($lswsHome = DALiteSpeedUtil::getLswsHome()) == '' ) {
            $lswsHome = self::VAL_LSWS_HOME_NOT_SET;
        }

        return DALiteSpeedControllerBase::ADMIN_CGI_DIR . '/'
            . (($asRoot) ? 'super_script' : 'lsadm_script') . ' '
            . escapeshellarg($lswsHome) . " $pid $cmd"
            . ((!empty($args)) ? " " . escapeshellarg(serialize($args)) : '')
        ;
    }

    /**
     *
     * @since 2.0
     *
     * @param bool   $asRoot
     * @param string $cmd
     * @param array  $args
     *
     * @return mixed|void
     *
     * @throws DAPluginException  Thrown when passed in $cmd is unsupported.
     * @throws DAPluginException  Thrown when getmypid() call returns boolean
     *     false.
     * @throws DAPluginException  Thrown indirectly by
     *     DAPluginLogger::processBuffer() call.
     * @throws DAPluginException  Thrown indirectly by
     *     DAPluginLogger::preCreateLogFile() call.
     * @throws DAPluginException  Thrown indirectly by
     *     self::handleTempEscalationLog() call.
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::debug()
     *     call.
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::debug()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     self::handleMgrCmdOutput() call.
     */
    public static function escalate( $asRoot, $cmd, array $args = array() )
    {
        if ( ($cmdType = self::getSupportedCmdType($cmd)) == '' ) {
            throw new DAPluginException("Illegal escalation command $cmd.");
        }

        if ( ($pid = getmypid()) === false ) {
            throw new DAPluginException(
                'Failed to get current PHP process ID when attempting to '
                    . 'escalate.'
            );
        }

        $execCmd = self::getEscalateCmd($pid, $asRoot, $cmd, $args);
        $output  = array();

        DALiteSpeedUtil::exec_cmd($execCmd, $output, $return_var);

        $tempEscalatedLog =
            DAPluginEscalatedLogger::TEMP_ESCALATED_LOG_BASE . "_$pid.log";

        if ( file_exists($tempEscalatedLog) ) {

            if ( $cmd != self::MGR_CMD_WRITE_TO_LOG ) {
                DAPluginLogger::processBuffer();
                DAPluginLogger::preCreateLogFile();
            }

            self::handleTempEscalationLog($pid);
        }

        DAPluginLogger::debug(
            "Escalation command $cmd = $return_var\n$execCmd"
        );
        DAPluginLogger::debug('output = ' . var_export($output, true));

        switch ($cmdType) {

            case self::TYPE_CMD_LSWS:
                return self::handleLswsCmdOutput($output);

            case self::TYPE_CMD_MGR:
                return self::handleMgrCmdOutput($output);

            //no default case
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param string[] $output
     *
     * @return string[]
     */
    private static function handleLswsCmdOutput( array $output )
    {
        $result = array();

        $msg = '';

        foreach ( $output as $line ) {

            if ( $line != '' && is_string($line) ) {

                if ( preg_match("/^RESULT (\w+)=(.+)$/", $line, $matches) ) {
                    $result[$matches[1]] = $matches[2];
                }
                else {
                    $msg .= "$line<br />";
                }
            }
        }

        if ( !empty($msg) ) {
            $result['MESG'] = $msg;
        }

        return $result;
    }

    /**
     *
     * @since 2.0
     *
     * @param string[] $output
     *
     * @return mixed
     *
     * @throws DAPluginException  Thrown when $err is not empty.
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::debug()
     *     call.
     */
    private static function handleMgrCmdOutput( array $output )
    {
        $ret = null;
        $err = '';

        foreach ( $output as $line ) {

            if ( ($pos = strpos($line, '[RESULT]')) !== false ) {
                $ret = unserialize(base64_decode(substr($line, $pos + 9)));

                DAPluginLogger::debug(
                    'Decoded [RESULT] = ' . var_export($ret, true)
                );
            }
            elseif ( ($pos = strpos($line, '[ERROR]')) !== false ) {
                $err .= substr($line, $pos + 7) . "\n";
            }
            elseif ( $line != '' || !empty($err) ) {
                $err .= "$line\n";
            }
        }

        if ( !empty($err) ) {
            throw new DAPluginException($err);
        }

        return $ret;
    }

    /**
     *
     * @since 2.0
     * @since 2.2  Added new parameter $pid.
     *
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::debug()
     *     call.
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::debug()
     *     call.
     */
    private static function handleTempEscalationLog( $pid )
    {
        $subAction = self::MGR_CMD_PROCESS_TEMP_ESCALATION_LOG;
        $subCmd    = self::getEscalateCmd($pid, true, $subAction);

        exec($subCmd, $subOutput, $subReturn_var);

        DAPluginLogger::debug(
            "Issue sub command $subAction=$subReturn_var\n$subCmd"
        );
        DAPluginLogger::debug('sub output = ' . var_export($subOutput, true));
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown when getmypid() call returns boolean
     *     false.
     * @throws DAPluginException  Thrown indirectly by
     *     "new DAPanelLsadmContextOption()" call.
     * @throws DAPluginException  Thrown indirectly by
     *     DAPluginEscalatedLogger::getLogFilePath() call.
     * @throws LSCMException      Thrown indirectly by Context::initialize()
     *     call.
     * @throws LSCMException      Thrown indirectly by Context::initialize()
     *     call.
     */
    private function loadSharedCode()
    {
        require_once DALiteSpeedUtil::getLswsHome()
            . '/add-ons/webcachemgr/autoloader.php';

        $panelName = 'directadmin';

        if ( posix_getuid() == 0 ) {
            $ctxOption = new RootPanelContextOption($panelName);
        }
        else {
            $ctxOption = new DAPanelLsadmContextOption($panelName);
        }

        $apiAtLeast_v1_9_1 = DALiteSpeedUtil::betterVersionCompare(
            ControlPanel::PANEL_API_VERSION,
            '1.9.1',
            '>='
        );

        if ( $apiAtLeast_v1_9_1 ) {
            Context::initialize(
                $ctxOption,
                DAPluginEscalatedLogger::getInstance()
            );
        }
        else {
            Context::initialize($ctxOption);

            $apiAtLeast_v1_9 = DALiteSpeedUtil::betterVersionCompare(
                ControlPanel::PANEL_API_VERSION,
                '1.9',
                '>='
            );

            if ( $apiAtLeast_v1_9 ) {

                if ( ($pid = getmypid()) === false ) {
                    throw new DAPluginException(
                        'Failed to get current PHP process ID when attempting '
                            . 'to set Logger additional tag info.'
                    );
                }

                Logger::changeLogFileUsed(
                    DAPluginEscalatedLogger::getLogFilePath()
                );

                if ( isset($_SERVER['REMOTE_ADDR']) ) {
                    $remoteAddr = $_SERVER['REMOTE_ADDR'];
                }
                else {
                    $remoteAddr = 'cli';
                }

                Logger::setAdditionalTagInfo("[$remoteAddr-$pid]");
            }
        }
    }

    /**
     *
     * @since 2.0
     *
     * @return array[]|bool|string|string[]|string[][]|null
     *
     * @throws DAPluginException  Thrown indirectly by $this->loadSharedCode()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->e2_daVhCacheRootSearch() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->e2_getCacheRootSetting() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->e2_createVHConfAndSetCacheRoot() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->e2_writeVHCacheRoot() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->e2_setActiveVersion() call.
     * @throws DAPluginException  Thrown indirectly by $this->e2_doAction()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->e2_dataFileExists() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->e2_getDataFileContent() call.
     * @throws DAPluginException  Thrown indirectly by $this->e2_scan2Batch()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->e2_updateDataFile() call.
     * @throws LSCMException  Thrown indirectly by $this->loadSharedCode() call.
     * @throws LSCMException  Thrown indirectly by
     *     $this->e2_daVhCacheRootSearch() call.
     * @throws LSCMException  Thrown indirectly by
     *     $this->e2_getCacheRootSetting() call.
     * @throws LSCMException  Thrown indirectly by $this->e2_isCacheEnabled()
     *     call.
     * @throws LSCMException  Thrown indirectly by
     *     $this->e2_createVHConfAndSetCacheRoot() call.
     * @throws LSCMException  Thrown indirectly by
     *     $this->e2_applyVHConfChanges() call.
     * @throws LSCMException  Thrown indirectly by
     *     $this->e2_getLSWSCacheRootSetting() call.
     * @throws LSCMException  Thrown indirectly by
     *     $this->e2_setServerCacheRoot() call.
     * @throws LSCMException  Thrown indirectly by $this->e2_writeVHCacheRoot()
     *     call.
     * @throws LSCMException  Thrown indirectly by
     *     $this->e2_refreshVersionList() call.
     * @throws LSCMException  Thrown indirectly by $this->e2_getCurrentVersion()
     *     call.
     * @throws LSCMException  Thrown indirectly by $this->e2_setActiveVersion()
     *     call.
     * @throws LSCMException  Thrown indirectly by $this->e2_getDocrootMap()
     *     call.
     * @throws LSCMException  Thrown indirectly by $this->e2_doAction() call.
     * @throws LSCMException  Thrown indirectly by $this->e2_dataFileExists()
     *     call.
     * @throws LSCMException  Thrown indirectly by
     *     $this->e2_getDataFileContent() call.
     * @throws LSCMException  Thrown indirectly by $this->e2_updateDataFile()
     *     call.
     */
    private function doSharedCodeDependentAction()
    {
        $this->loadSharedCode();

        switch ($this->action) {

            case self::MGR_CMD_DA_VH_CACHE_ROOT_SEARCH:
                return $this->e2_daVhCacheRootSearch();

            case self::MGR_CMD_GET_CACHE_ROOT_SETTING:
                return $this->e2_getCacheRootSetting();

            case self::MGR_CMD_IS_CACHE_ENABLED:
                return $this->e2_isCacheEnabled();

            case self::MGR_CMD_GET_LSWS_CACHE_ROOT_SETTING:
                return $this->e2_getLSWSCacheRootSetting();

            case self::MGR_CMD_GET_CURR_VER:
                return $this->e2_getCurrentVersion();

            case self::MGR_CMD_GET_DOCROOT_MAP:
                return $this->e2_getDocrootMap();

            case self::MGR_CMD_DO_ACTION:
                return $this->e2_doAction();

            case self::MGR_CMD_DATA_FILE_EXISTS:
                return $this->e2_dataFileExists();

            case self::MGR_CMD_GET_DATA_FILE_CONTENT:
                return $this->e2_getDataFileContent();

            case self::MGR_CMD_GET_WP_DASH_MSG_DATA:
                return $this->e2_getWpDashMsgData();

            case self::MGR_CMD_SCAN_2_BATCH:
                return $this->e2_scan2Batch();

            case self::MGR_CMD_UPDATE_DATA_FILE:
                return $this->e2_updateDataFile();

            case self::MGR_CMD_CREATE_VHCONF_AND_SET_CACHE_ROOT:
                $this->e2_createVHConfAndSetCacheRoot();
                break;

            case self::MGR_CMD_APPLY_VHCONF_CHANGES:
                $this->e2_applyVHConfChanges();
                break;

            case self::MGR_CMD_POPULATE_MIN_API_VER_FILE:
                $this->e2_populateMinAPIVerFile();
                break;

            case self::MGR_CMD_SET_SERVER_CACHE_ROOT:
                $this->e2_setServerCacheRoot();
                break;

            case self::MGR_CMD_WRITE_VH_CACHE_ROOT:
                $this->e2_writeVHCacheRoot();
                break;

            case self::MGR_CMD_REFRESH_VER_LIST:
                $this->e2_refreshVersionList();
                break;

            case self::MGR_CMD_SET_ACTIVE_VER:
                $this->e2_setActiveVersion();
                break;

            //no default
        }

        return null;
    }

    /**
     *
     * @since 2.2
     *
     * @return array[]|bool|int|string|string[]|string[][]|null
     *
     * @throws DAPluginException  Thrown when LSWS_HOME value is not set.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->doSharedCodeDependentAction() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->e2_writeWpDashMsgsData() call.
     * @throws DAPluginException  Thrown indirectly by $this->e2_switchLicense()
     *     call.
     * @throws LSCMException  Thrown indirectly by
     *     $this->doSharedCodeDependentAction() call.
     */
    private function doLswsHomeRequiredAction()
    {
        if ( DALiteSpeedUtil::getLswsHome() == '' ) {
            throw new DAPluginException(
                'LSWS_HOME value not set when attempting to execute LSWS_HOME '
                    . "required command '$this->action'."
            );
        }

        switch ( $this->action ) {

            case self::MGR_CMD_APPLY_VHCONF_CHANGES:
            case self::MGR_CMD_CREATE_VHCONF_AND_SET_CACHE_ROOT:
            case self::MGR_CMD_DA_VH_CACHE_ROOT_SEARCH:
            case self::MGR_CMD_DATA_FILE_EXISTS:
            case self::MGR_CMD_DO_ACTION:
            case self::MGR_CMD_GET_CACHE_ROOT_SETTING:
            case self::MGR_CMD_GET_CURR_VER:
            case self::MGR_CMD_GET_DATA_FILE_CONTENT:
            case self::MGR_CMD_GET_DOCROOT_MAP:
            case self::MGR_CMD_GET_LSWS_CACHE_ROOT_SETTING:
            case self::MGR_CMD_GET_WP_DASH_MSG_DATA:
            case self::MGR_CMD_IS_CACHE_ENABLED:
            case self::MGR_CMD_POPULATE_MIN_API_VER_FILE:
            case self::MGR_CMD_REFRESH_VER_LIST:
            case self::MGR_CMD_SCAN_2_BATCH:
            case self::MGR_CMD_SET_ACTIVE_VER:
            case self::MGR_CMD_SET_SERVER_CACHE_ROOT:
            case self::MGR_CMD_UPDATE_DATA_FILE:
            case self::MGR_CMD_WRITE_VH_CACHE_ROOT:
                return $this->doSharedCodeDependentAction();

            case self::MGR_CMD_UPDATE_SHARED_CODE:
                return $this->e2_updateSharedCode();

            case self::MGR_CMD_WRITE_WP_DASH_MSGS_DATA:
                return $this->e2_writeWpDashMsgsData();

            case self::LSWS_CMD_GET_LICENSE_STATUS:
                $this->e2_echoCurrentLicenseStatus();
                break;

            case self::LSWS_CMD_SWITCH_LICENSE:
                $this->e2_switchLicense();
                break;

            //no default
        }

        return null;
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown when getmypid() call returns boolean
     *     false.
     * @throws DAPluginException  Re-thrown when a DAPluginException with code
     *     self::EXIT_SUCC is caught.
     * @throws DAPluginException  Thrown when a DAPluginException without code
     *     self::EXIT_SUCC is caught.
     * @throws DAPluginException  Thrown when an LSCMException is caught.
     * @throws DAPluginException  Thrown indirectly by
     *     DAPluginEscalatedLogger::InitializeWithParentPid() call.
     * @throws DAPluginException  Thrown indirectly by
     *     DAPluginEscalatedLogger::setAdditionalTagInfo() call.
     */
    private function runEscalatedCmd()
    {

        if ( ($pid = getmypid()) === false ) {
            throw new DAPluginException(
                '[ERROR] Failed to get current PHP process ID when attempting '
                    . "to set DAPluginEscalatedLogger additional tag info.\n",
                self::EXIT_ERROR
            );
        }

        DAPluginEscalatedLogger::InitializeWithParentPid($this->parentPid);

        if ( isset($_SERVER['REMOTE_ADDR']) ) {
            $remoteAddr = $_SERVER['REMOTE_ADDR'];
        }
        else {
            $remoteAddr = 'cli';
        }

        DAPluginEscalatedLogger::setAdditionalTagInfo("[$remoteAddr-$pid]");

        $ret = null;

        try {
            switch ($this->action) {

                case self::LSWS_CMD_GET_LICENSE_STATUS:
                case self::LSWS_CMD_SWITCH_LICENSE:
                case self::MGR_CMD_APPLY_VHCONF_CHANGES:
                case self::MGR_CMD_CREATE_VHCONF_AND_SET_CACHE_ROOT:
                case self::MGR_CMD_DA_VH_CACHE_ROOT_SEARCH:
                case self::MGR_CMD_DATA_FILE_EXISTS:
                case self::MGR_CMD_DO_ACTION:
                case self::MGR_CMD_GET_CACHE_ROOT_SETTING:
                case self::MGR_CMD_GET_CURR_VER:
                case self::MGR_CMD_GET_DATA_FILE_CONTENT:
                case self::MGR_CMD_GET_DOCROOT_MAP:
                case self::MGR_CMD_GET_LSWS_CACHE_ROOT_SETTING:
                case self::MGR_CMD_GET_WP_DASH_MSG_DATA:
                case self::MGR_CMD_IS_CACHE_ENABLED:
                case self::MGR_CMD_POPULATE_MIN_API_VER_FILE:
                case self::MGR_CMD_REFRESH_VER_LIST:
                case self::MGR_CMD_SCAN_2_BATCH:
                case self::MGR_CMD_SET_ACTIVE_VER:
                case self::MGR_CMD_SET_SERVER_CACHE_ROOT:
                case self::MGR_CMD_UPDATE_DATA_FILE:
                case self::MGR_CMD_UPDATE_SHARED_CODE:
                case self::MGR_CMD_WRITE_VH_CACHE_ROOT:
                case self::MGR_CMD_WRITE_WP_DASH_MSGS_DATA:
                    $ret = $this->doLswsHomeRequiredAction();
                    break;

                case self::MGR_CMD_REMOVE_FILE:
                    $ret = $this->e2_removeFile();
                    break;

                case self::MGR_CMD_WRITE_DASH_PLUGIN_MD5:
                    $ret = $this->e2_writeDashPluginMd5();
                    break;

                case self::LSWS_CMD_GET_MAINMENU_INFO:
                    $this->e1_echoMainMenuInfo();
                    break;

                case self::LSWS_CMD_GET_SERIAL_NO:
                    $this->e1_echoCurrentSerialNo();
                    break;

                case self::LSWS_CMD_RESTART_LSWS:
                    $this->e2_restartLsws();
                    break;

                case self::MGR_CMD_WRITE_TO_LOG:
                    $ret = $this->e2_writeToLog();
                    break;

                case self::MGR_CMD_GET_REAL_PATH:
                    $ret = $this->e2_realPath();
                    break;

                case self::MGR_CMD_PROCESS_TEMP_ESCALATION_LOG:
                    $ret = $this->e2_processTempEscalationLog();
                    break;

                case self::MGR_CMD_FILE_EXISTS:
                    $ret = $this->e2_fileExists();
                    break;

                // no default case
            }

            $msg = '';

            if ( $ret !== null ) {
                $msg = '[RESULT] ' . base64_encode(serialize($ret)) . "\n";
            }

            throw new DAPluginException($msg, self::EXIT_SUCC);
        }
        catch ( DAPluginException $e ) {

            /**
             * Re-throw success.
             */
            if ( $e->getCode() == self::EXIT_SUCC ) {
                throw $e;
            }

            throw new DAPluginException(
                "[ERROR] {$e->getMessage()}\n",
                self::EXIT_ERROR
            );
        }
        catch ( LSCMException $e ) {
            throw new DAPluginException(
                "[ERROR] {$e->getMessage()}\n",
                self::EXIT_ERROR
            );
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param string $cmd
     *
     * @return string Command type or empty string if a matching command is
     *     not found.
     */
    private static function getSupportedCmdType( $cmd )
    {
        $isLswsCmd = in_array(
            $cmd,
            array(
                self::LSWS_CMD_GET_LICENSE_STATUS,
                self::LSWS_CMD_GET_MAINMENU_INFO,
                self::LSWS_CMD_GET_SERIAL_NO,
                self::LSWS_CMD_RESTART_LSWS,
                self::LSWS_CMD_SWITCH_LICENSE
            )
        );

        if ( $isLswsCmd ) {
            return self::TYPE_CMD_LSWS;
        }

        $isMgrCmd = in_array(
            $cmd,
            array(
                self::MGR_CMD_APPLY_VHCONF_CHANGES,
                self::MGR_CMD_CREATE_VHCONF_AND_SET_CACHE_ROOT,
                self::MGR_CMD_DA_VH_CACHE_ROOT_SEARCH,
                self::MGR_CMD_DO_ACTION,
                self::MGR_CMD_DATA_FILE_EXISTS,
                self::MGR_CMD_FILE_EXISTS,
                self::MGR_CMD_GET_CACHE_ROOT_SETTING,
                self::MGR_CMD_GET_CURR_VER,
                self::MGR_CMD_GET_DOCROOT_MAP,
                self::MGR_CMD_GET_DATA_FILE_CONTENT,
                self::MGR_CMD_GET_LSWS_CACHE_ROOT_SETTING,
                self::MGR_CMD_GET_REAL_PATH,
                self::MGR_CMD_GET_WP_DASH_MSG_DATA,
                self::MGR_CMD_IS_CACHE_ENABLED,
                self::MGR_CMD_POPULATE_MIN_API_VER_FILE,
                self::MGR_CMD_PROCESS_TEMP_ESCALATION_LOG,
                self::MGR_CMD_REFRESH_VER_LIST,
                self::MGR_CMD_REMOVE_FILE,
                self::MGR_CMD_SCAN_2_BATCH,
                self::MGR_CMD_SET_ACTIVE_VER,
                self::MGR_CMD_SET_SERVER_CACHE_ROOT,
                self::MGR_CMD_UPDATE_DATA_FILE,
                self::MGR_CMD_UPDATE_SHARED_CODE,
                self::MGR_CMD_WRITE_DASH_PLUGIN_MD5,
                self::MGR_CMD_WRITE_TO_LOG,
                self::MGR_CMD_WRITE_VH_CACHE_ROOT,
                self::MGR_CMD_WRITE_WP_DASH_MSGS_DATA,
            )
        );

        if ( $isMgrCmd ) {
            return self::TYPE_CMD_MGR;
        }

        return '';
    }

    /**
     * This function should only be executed when this script is included
     * through the DAEscalateBootstrap.php script after escalating with the
     * appropriate escalation script.
     *
     * @since 2.0
     *
     * @param string   $caller
     * @param string[] $input
     *
     * @return void
     *
     * @throws DAPluginException  Thrown indirectly by "new self()" call.
     * @throws DAPluginException  Thrown indirectly by
     *     $daEscalate->runEscalatedCmd() call.
     */
    public static function run( $caller, array $input )
    {
        /**
         * Only run if called from bootstrap file.
         */
        if ( $caller != self::BOOTSTRAP_FILE ) {
            return;
        }

        date_default_timezone_set('UTC');

        $daEscalate = new self($input);
        $daEscalate->runEscalatedCmd();
    }

}
