<?php

/** ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019-2023
 * @since 2.0
 * ******************************************* */

namespace LsPanel;

use Lsc\Wp\LSCMException;
use Lsc\Wp\Panel\ControlPanel;
use LsPanel\Extend\DirectAdminPanel;
use LsPanel\View\Model as DAViewModel;

/**
 * @since 2.0
 */
class DALiteSpeedController extends DALiteSpeedControllerBase
{

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown indirectly by parent::__construct()
     *     call.
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     *
     * @since 2.3
     *
     * @param null|string $do
     *
     * @return bool
     */
    private static function isSupportedAction( $do )
    {
        return in_array(
            $do,
            array (
                'restart_lsws',
                'check_current_license',
                'change_license',
                'transfer_license',
                'lscwp_manage',
                'cacheRootSetup',
                'lscwp_mass_enable_disable',
                'lscwpVersionManager',
                'dash_notifier',
                'dash_slug_verify',
                'updateSharedCode',
                'main',
                ''
            )
        );
    }

    /**
     *
     * @since 2.0
     * @since 2.1  Added optional parameters $do, $step, $mgrStep, and
     *     $dashStep.
     *
     * @param null|string $do
     * @param null|int    $step
     * @param null|int    $mgrStep
     * @param null|int    $dashStep
     *
     * @throws DAPluginException  Thrown when values for $do, $step, $mgrStep,
     *     or $dashStep are not recognized.
     * @throws DAPluginException  Thrown indirectly by $this->restartLsws()
     *     call.
     * @throws DAPluginException  Thrown indirectly by $this->change_license()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->doSharedCodeDependentAction() call.
     * @throws DAPluginException  Thrown indirectly by $this->run() call.
     * @throws DAPluginException  Thrown indirectly by $this->dashSlugVerify()
     *     call.
     * @throws DAPluginException  Thrown indirectly by $this->updateSharedCode()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->getMainMenuViewModel() call.
     * @throws DAPluginException  Thrown indirectly by $this->display() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->handleDefaultAjaxResponse() call.
     */
    public function run(
        $do = null,
        $step = null,
        $mgrStep = null,
        $dashStep = null )
    {
        parent::setRunRequestVars($do, $step, $mgrStep, $dashStep);

        if ( !self::isSupportedAction($do)
                || !in_array($step, parent::RANGE_STEP)
                || !in_array($mgrStep, parent::RANGE_MGR_STEP)
                || !in_array($dashStep, parent::RANGE_DASH_STEP) ) {

            throw new DAPluginException('Invalid Entrance');
        }

        ob_start();

        switch ($do) {

            case 'restart_lsws':
                $viewModel = $this->restartLsws($step);
                break;

            case 'check_current_license':
                $viewModel = new DAViewModel\CurrentLicenseStatusViewModel();
                break;

            case 'change_license':
                $viewModel = $this->change_license($step);
                break;

            case 'transfer_license':
                $viewModel = new DAViewModel\TransferLicenseViewModel();
                break;

            case 'lscwp_manage':
            case 'cacheRootSetup':
            case 'lscwpVersionManager':
            case 'lscwp_mass_enable_disable':
            case 'dash_notifier':
                $ret = $this->doSharedCodeDependentAction(
                    $do,
                    $step,
                    $mgrStep,
                    $dashStep
                );

                if ( $ret == self::REDIRECT_TO_MAIN_MENU ) {
                    $this->run('main', 0, 0, 0);
                    return;
                }

                $viewModel = $ret;
                break;

            case 'dash_slug_verify':
                $this->dashSlugVerify();
                break;

            case 'updateSharedCode':
                $viewModel = $this->updateSharedCode();
                break;

            case 'main':
            case '':
                $viewModel = $this->getMainMenuViewModel();
                break;

            //no default
        }

        if ( !empty($viewModel) ) {
            $this->display($viewModel);
        }

        if ( defined('LSWS_AJAX_CALL') && LSWS_AJAX_CALL ) {
            $this->handleDefaultAjaxResponse(ob_get_clean());
        }

        ob_flush();
    }

    /**
     *
     * @since 2.0
     *
     * @param string $do
     * @param int    $step
     * @param int    $mgrStep
     * @param int    $dashStep
     *
     * @return object|string|null  ViewModel object, a redirection string, or
     *     null if called with an improper $do value.
     *
     * @throws DAPluginException  Thrown indirectly by $this->lscwpManage()
     *     call.
     * @throws DAPluginException  Thrown indirectly by $this->cacheRootSetup()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->lscwpVersionManager() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->lscwpMassEnableDisableControl() call.
     * @throws DAPluginException  Thrown indirectly by $this->dashNotifier()
     *     call.
     */
    private function doSharedCodeDependentAction(
        $do,
        $step,
        $mgrStep,
        $dashStep )
    {
        if ( ! DALiteSpeedUtil::sharedCodeLoaded()
                || DALiteSpeedUtil::betterVersionCompare(ControlPanel::PANEL_API_VERSION, '1.9.7', '<')
                || ! DirectAdminPanel::meetsMinAPIVerRequirement() ) {

            return (new DAViewModel\SharedCodeUpdateRequiredViewModel());
        }

        $panelAPICompatStatus =
            DirectAdminPanel::checkPanelAPICompatibility(self::SHARED_API_VER);

        switch ( $panelAPICompatStatus ) {

            case ControlPanel::PANEL_API_VERSION_TOO_HIGH:
                return (new DAViewModel\SharedCodeUpdateRequiredViewModel());

            case ControlPanel::PANEL_API_VERSION_TOO_LOW:
            case ControlPanel::PANEL_API_VERSION_UNKNOWN:
                return (new DAViewModel\PluginUpdateRequiredViewModel());

            //no default
        }

        switch ($do) {

            case 'lscwp_manage':
                return $this->lscwpManage($mgrStep);

            case 'cacheRootSetup':
                return $this->cacheRootSetup($step);

            case 'lscwpVersionManager':
                return $this->lscwpVersionManager($step);

            case 'lscwp_mass_enable_disable':
                return $this->lscwpMassEnableDisableControl($step);

            case 'dash_notifier':
                return $this->dashNotifier($dashStep);

            default:
                return null;
        }
    }

    /**
     *
     * @since 2.0
     *
     * @return DAViewModel\MainMenuViewModel
     *
     * @throws DAPluginException  Thrown indirectly by
     * "new DAViewModel\MainMenuViewModel()" call.
     *
     * @noinspection PhpDocRedundantThrowsInspection
     *
     */
    protected function getMainMenuViewModel()
    {
        /**
         * Throws DAPluginException.
         */
        return (new DAViewModel\MainMenuViewModel());
    }

    /**
     *
     * @since 2.0
     *
     * @param int $step
     *
     * @return DAViewModel\BaseViewModel
     *
     * @throws DAPluginException  Thrown indirectly by
     *     DALiteSpeedUtil::switchLicense() call.
     */
    private function change_license( $step )
    {
        $info = array(
            'errors' => array(),
            'install_type' => '',
            'license_agree' => '',
            'serial_no' => ''
        );

        if ( $step == 1 ) {
            DALiteSpeedUtil::validateLicenseChangeInput($info);

            if ( empty($info['errors']) ) {
                DALiteSpeedUtil::switchLicense($info);

                return (new DAViewModel\ChangeLicenseViewModel($info));
            }
        }

        return (new DAViewModel\ChangeLicenseConfirmViewModel($info));
    }

    /**
     *
     * @since 2.0
     *
     * @param int $step
     *
     * @return DAViewModel\CacheRootSetupViewModel
     *
     * @throws DAPluginException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::error()
     *     call.
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::error()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     "new DAViewModel\CacheRootSetupViewModel()" call.
     */
    private function cacheRootSetup( $step )
    {
        $panelEnv = $this->getControlPanelInstance();

        if ( $step == 1 ) {

            try {
                $panelEnv->verifyCacheSetup();
            }
            catch ( LSCMException $e ) {
                DAPluginLogger::error($e->getMessage());
                DAPluginLogger::uiError($e->getMessage());
            }
        }

        return (new DAViewModel\CacheRootSetupViewModel($panelEnv));
    }

}
