<?php

/** ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019-2023
 * @since 2.0
 * ******************************************* */

namespace LsPanel;

use Lsc\Wp\Context\Context;
use Lsc\Wp\LSCMException;
use Lsc\Wp\PanelController;
use Lsc\Wp\PluginVersion;
use Lsc\Wp\View\Model as LscViewModel;
use LsPanel\Extend\DAPanelController;
use LsPanel\Extend\DAPanelWPDashMsgs;
use LsPanel\Extend\DAPanelWPInstallStorage;
use LsPanel\Extend\DirectAdminPanel;
use LsPanel\View\Model as DAViewModel;
use LsPanel\View\View;

/**
 * @since 2.0
 */
abstract class DALiteSpeedControllerBase
{

    /**
     * @var string
     */
    const ADMIN_CGI_DIR = '/usr/local/directadmin/plugins/litespeed/admin';

    /**
     * @var string
     */
    const MODULE_VERSION = '2.3.1.1';

    /**
     * @since 2.1
     * @var string
     */
    const REDIRECT_TO_MAIN_MENU = 'redirectToMainMenu';

    /**
     * @var string
     */
    const SHARED_API_VER = '1.15';

    /**
     * @since 2.2
     * @var int[]
     */
    const RANGE_STEP = array( 0, 1, 2, 3 );

    /**
     * @since 2.2
     * @var int[]
     */
    const RANGE_MGR_STEP = array( 0, 1, 2, 3, 4 );

    /**
     * @since 2.2
     * @var int[]
     */
    const RANGE_DASH_STEP = array( 0, 1, 2 );

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Rethrown when Context::getoption() call throws
     *     an LSCMException exception.
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::debug()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     DALiteSpeedUtil::initialize() call.
     */
    public function __construct()
    {
        if ( DALiteSpeedUtil::setLswsHome() ) {

            if ( $this->includeWebCacheMgrBootstrap() ) {
                DALiteSpeedUtil::setSharedCodeLoaded(true);

                try {
                    Context::getoption()->setIconDir(
                        './litespeed/images/icons'
                    );
                }
                catch ( LSCMException $e ) {
                    $msg = "{$e->getMessage()} Unable to set icon directory.";
                    DAPluginLogger::debug($msg);

                    throw new DAPluginException($msg);
                }
            }
        }

        DALiteSpeedUtil::initialize();
    }

    /**
     *
     * @since 2.2
     *
     * @param null|string $do
     * @param null|int    $step
     * @param null|int    $mgrStep
     * @param null|int    $dashStep
     */
    protected static function setRunRequestVars(
        &$do,
        &$step,
        &$mgrStep,
        &$dashStep )
    {
        if ( $do === null ) {
            $do = DALiteSpeedUtil::get_request_var('do');
        }

        if ( $step === null ) {
            $step = DALiteSpeedUtil::get_request_var('step', 0);
        }

        if ( $mgrStep === null ) {
            $mgrStep = DALiteSpeedUtil::get_request_var('mgr_step', 0);
        }

        if ( $dashStep === null ) {
            $dashStep = DALiteSpeedUtil::get_request_var('dash_step', 0);
        }
    }

    /**
     *
     * @since 2.0
     */
    abstract public function run();

    /**
     * Output's an Ajax response with a default 'Content-Type' of 'text/html'
     * when an Ajax request has been made and a response is not explicitly sent
     * elsewhere in the code.
     *
     * @since 2.0
     *
     * @param string $ajaxContent
     *
     * @throws DAPluginException  Thrown indirectly by
     *     DAPluginAjaxResponse::outputAndExit() call.
     */
    protected function handleDefaultAjaxResponse( $ajaxContent )
    {
        DAPluginAjaxResponse::setHeaderContent(
            "HTTP/1.1 200 OK\nContent-Type: text/html"
        );
        DAPluginAjaxResponse::setAjaxContent($ajaxContent);
        DAPluginAjaxResponse::outputAndExit();
    }

    /**
     *
     * @since 2.0
     *
     * @param string $msg
     *
     * @throws DAPluginException  Thrown indirectly by $view->display() call.
     */
    protected function displayMissingTpl( $msg )
    {
        $view = new View(new DAViewModel\MissingTplViewModel($msg));
        $view->display();
    }

    /**
     * Attempt to include shared code.
     *
     * @since 2.0
     *
     * @return bool
     */
    protected function includeWebCacheMgrBootstrap()
    {
        $lswsHome = DALiteSpeedUtil::getLswsHome();

        if ( ! file_exists("$lswsHome/add-ons/webcachemgr/autoloader.php")
                || ! include_once __DIR__ . '/webCacheMgrBootstrap.php' ) {

            return false;
        }

        return true;
    }

    /**
     *
     * @since 2.0
     *
     * @return object  ViewModel object.
     *
     * @throws DAPluginException  Thrown indirectly by
     *     DALiteSpeedUtil::updateSharedCode() call.
     */
    protected function updateSharedCode()
    {
        return (
            new DAViewModel\SharedCodeUpdateResultViewModel(
                DALiteSpeedUtil::newUpdateSharedCode()
            )
        );
    }

    /**
     *
     * @since 2.0
     *
     * @return object  ViewModel object.
     */
    abstract protected function getMainMenuViewModel();


    /**
     *
     * @since 2.0
     *
     * @param int $step
     *
     * @return object  ViewModel object.
     *
     * @throws DAPluginException  Thrown indirectly by
     *     DALiteSpeedUtil::restartLSWS() call.
     */
    protected function restartLsws( $step )
    {
        if ( $step == 1 ) {
            DALiteSpeedUtil::restartLSWS();

            return (new DAViewModel\RestartLswsViewModel());
        }

        return (new DAViewModel\RestartLswsConfirmViewModel());
    }

    /**
     *
     * @since 2.0
     *
     * @return DirectAdminPanel
     *
     * @throws DAPluginException  Rethrown when
     *     DirectAdminPanel::getClassInstance() call throws an LSCMException
     *     exception.
     */
    protected function getControlPanelInstance()
    {
        try {
            /**
             * @var DirectAdminPanel $panel
             */
            $panel = DirectAdminPanel::getClassInstance(
                '\LsPanel\Extend\DirectAdminPanel'
            );
        }
        catch ( LSCMException $e ) {
            throw new DAPluginException($e->getMessage(), $e->getCode());
        }

        return $panel;
    }

    /**
     *
     * @since 2.3
     *
     * @return DAPanelWPInstallStorage
     *
     * @throws DAPluginException  Rethrown when Context::getLSCMDataFiles() or
     *     "new DAPanelWPInstallStorage()" calls throw an LSCMException
     *     exception.
     */
    protected function getDAPanelWPInstallStorageInstance()
    {
        try {
            $lscmDataFiles = Context::getLSCMDataFiles();

            return (
                new DAPanelWPInstallStorage(
                    $lscmDataFiles['dataFile'],
                    $lscmDataFiles['custDataFile']
                )
            );
        }
        catch ( LSCMException $e ) {
            throw new DAPluginException($e->getMessage(), $e->getCode());
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param int $mgrStep
     *
     * @return object  ViewModel object.
     *
     * @throws DAPluginException  Rethrown when $panelEnv->areCacheRootsSet()
     *     call throws an LSCMException exception.
     * @throws DAPluginException  Rethrown when PluginVersion::getInstance() or
     *     $panelController->manageCacheOperations2() calls throw an
     *     LSCMException exception.
     * @throws DAPluginException  Rethrown when
     *     "new LscViewModel\ScanProgressViewModel()" call throws an
     *     LSCMException exception.
     * @throws DAPluginException  Rethrown when
     *     "new LscViewModel\ManageViewModel()" call throws an LSCMException
     *     exception.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     * @throws DAPluginException  Thrown indirectly by
     *     self::getDAPanelWPInstallStorageInstance() call.
     */
    protected function lscwpManage( $mgrStep )
    {
        $panelEnv = $this->getControlPanelInstance();

        try {
            if ( !$panelEnv->areCacheRootsSet() ) {
                return (new LscViewModel\CacheRootNotSetViewModel());
            }
        }
        catch ( LSCMException $e ) {
            throw new DAPluginException($e->getMessage(), $e->getCode());
        }

        $model = self::getDAPanelWPInstallStorageInstance();

        $panelController = new DAPanelController($panelEnv, $model, $mgrStep);

        try {
            /**
             * Pre-initialize DAPanelPluginVersion instance for viewModel.
             */
            PluginVersion::getInstance();

            $newMgrStep = $panelController->manageCacheOperations2();
        }
        catch ( LSCMException $e ) {
            throw new DAPluginException($e->getMessage(), $e->getCode());
        }

        switch ($newMgrStep) {

            case PanelController::MGR_STEP_SCAN:
            case PanelController::MGR_STEP_DISCOVER_NEW:

                try {
                    return (
                        new LscViewModel\ScanProgressStepViewModel($newMgrStep)
                    );
                }
                catch ( LSCMException $e ) {
                    throw new DAPluginException(
                        $e->getMessage(),
                        $e->getCode()
                    );
                }

            case PanelController::MGR_STEP_REFRESH_STATUS:
                return (new LscViewModel\RefreshStatusProgressViewModel());

            case PanelController::MGR_STEP_MASS_UNFLAG:
                return (new LscViewModel\UnflagAllProgressViewModel());

            default:

                try {
                    return (new LscViewModel\ManageViewModel($model));
                }
                catch ( LSCMException $e ) {
                    throw new DAPluginException(
                        $e->getMessage(),
                        $e->getCode()
                    );
                }
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param int $step
     *
     * @return object|string  ViewModel object or redirection string.
     *
     * @throws DAPluginException  Rethrown when $panelEnv->areCacheRootsSet()
     *     call throws an LSCMException exception.
     * @throws DAPluginException  Thrown indirectly by
     *     self::getDAPanelWPInstallStorageInstance() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->lscwpMassEnableDisable() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     */
    protected function lscwpMassEnableDisableControl( $step )
    {
        $model = self::getDAPanelWPInstallStorageInstance();

        if ( $step == 1 || $step == 2 ) {
            $action = DALiteSpeedUtil::get_request_var('act');

            if ( in_array($action, array( 'enable', 'disable' )) ) {
                return $this->lscwpMassEnableDisable($action, $step, $model);
            }
        }
        elseif ( $step == 0 ) {

            if ( $model->getError() != 0 ) {
                return (new LscViewModel\DataFileMsgViewModel($model));
            }

            $panelEnv = $this->getControlPanelInstance();

            try {
                if ( !$panelEnv->areCacheRootsSet() ) {
                    return (new LscViewModel\CacheRootNotSetViewModel());
                }
            }
            catch ( LSCMException $e ) {
                throw new DAPluginException($e->getMessage(), $e->getCode());
            }
        }

        return (new LscViewModel\MassEnableDisableViewModel($model));
    }

    /**
     *
     * @since 2.0
     *
     * @param string                  $action
     * @param int                     $step
     * @param DAPanelWPInstallStorage $model
     *
     * @return object|string  ViewModel object or redirection string.
     *
     * @throws DAPluginException  Rethrown when
     *     $panelController->massEnableDisable() call throws an LSCMException
     *     exception.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     */
    protected function lscwpMassEnableDisable(
                                $action,
                                $step,
        DAPanelWPInstallStorage $model )
    {
        if ( $model->getError() != 0 ) {
            return self::REDIRECT_TO_MAIN_MENU;
        }

        $panelController =
            new DAPanelController($this->getControlPanelInstance(), $model);

        try {
            $panelController->massEnableDisable($action, $step);
        }
        catch ( LSCMException $e ) {
            throw new DAPluginException($e->getMessage(), $e->getCode());
        }

        return (new LscViewModel\MassEnableDisableProgressViewModel($action));
    }

    /**
     *
     * @param int $step
     *
     * @return object  View model object.
     *
     * @throws DAPluginException  Rethrown when
     *     "new LscViewModel\VersionManageViewModel()" call throws an
     *     LSCMException exception.
     * @throws DAPluginException  Rethrown when
     *     $panelController->prepVersionChange() call throws an LSCMException
     *     exception.
     * @throws DAPluginException  Rethrown when
     *     "new LscViewModel\VersionManageViewModel()" call throws an
     *     LSCMException exception.
     * @throws DAPluginException  Thrown indirectly by
     *     self::getDAPanelWPInstallStorageInstance() call.
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::error()
     *     call.
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::uiError()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     */
    protected function lscwpVersionManager( $step )
    {
        $model = self::getDAPanelWPInstallStorageInstance();

        if ( $step == 2 || $step == 3 ) {
            $action = DALiteSpeedUtil::get_request_var('act');

            switch ($action) {
                case 'switchTo':
                    $ver_num = DALiteSpeedUtil::get_request_var('version_num');

                    try {
                        PluginVersion::getInstance()->setActiveVersion(
                            $ver_num
                        );
                    }
                    catch ( LSCMException $e ) {
                        $msg = $e->getMessage()
                            . " Could not switch active version to v$ver_num";

                        DAPluginLogger::error($msg);
                        DAPluginLogger::uiError($msg);
                    }

                    try {
                        return (new LscViewModel\VersionManageViewModel($model));
                    }
                    catch ( LSCMException $e ) {
                        throw new DAPluginException(
                            $e->getMessage(),
                            $e->getCode()
                        );
                    }

                case 'upgradeTo':
                    $panelController = new DAPanelController(
                        $this->getControlPanelInstance(),
                        $model
                    );

                    try {
                        $panelController->prepVersionChange($step);
                    }
                    catch ( LSCMException $e ) {
                        throw new DAPluginException(
                            $e->getMessage(),
                            $e->getCode()
                        );
                    }

                    return (new LscViewModel\VersionChangeViewModel());
            }
        }

        try {
            return (new LscViewModel\VersionManageViewModel($model));
        }
        catch ( LSCMException $e ) {
            throw new DAPluginException($e->getMessage(), $e->getCode());
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param int $dashStep
     *
     * @return object  ViewModel object.
     *
     * @throws DAPluginException  Rethrown when
     *     $panelController->manageDashOperations() call throws an LSCMException
     *     exception.
     * @throws DAPluginException  Thrown indirectly by
     *     self::getDAPanelWPInstallStorageInstance() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     */
    protected function dashNotifier( $dashStep )
    {
        $wpInstallStorage = self::getDAPanelWPInstallStorageInstance();

        $panelController = new DAPanelController(
            $this->getControlPanelInstance(),
            $wpInstallStorage
        );

        $wpDashMsgs = new DAPanelWPDashMsgs();

        try {
            $dashStep =
                $panelController->manageDashOperations($dashStep, $wpDashMsgs);
        }
        catch ( LSCMException $e ) {
            throw new DAPluginException($e->getMessage(), $e->getCode());
        }

        switch ( $dashStep ) {

            case PanelController::DASH_STEP_MASS_DASH_NOTIFY:
                return (new LscViewModel\MassDashNotifyProgressViewModel());

            case PanelController::DASH_STEP_MASS_DASH_DISABLE:
                return (new LscViewModel\MassDashDisableProgressViewModel());

            default:
                return (
                    new LscViewModel\DashNotifierViewModel(
                        $wpInstallStorage,
                        $wpDashMsgs
                    )
                );
        }
    }

    /**
     * Handle slug verification server side to bypass same domain policy
     * restrictions.
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown indirectly by
     *     DAPluginAjaxResponse::outputAndExit() call.
     */
    protected function dashSlugVerify()
    {
        $slug = DALiteSpeedUtil::get_request_var('slug');

        if ( $slug == NULL ) {
            $jsonData = '';
        }
        else {
            /**
             * $jsonData should already be in JSON format.
             */
            $jsonData = DALiteSpeedUtil::getUrlContents(
                filter_var(
                    "https://api.wordpress.org/plugins/info/1.0/$slug.json",
                    FILTER_SANITIZE_URL
                )
            );
        }

        DAPluginAjaxResponse::setHeaderContent(
            "HTTP/1.1 200 OK\nContent-Type: application/json"
        );
        DAPluginAjaxResponse::setAjaxContent($jsonData);
        DAPluginAjaxResponse::outputAndExit();
    }

    /**
     * Creates a View object with the passes ViewModel and displays to screen.
     *
     * @since 2.0
     *
     * @param object $viewModel  ViewModel object.
     *
     * @throws DAPluginException  Thrown indirectly by
     *     $this->displayMissingTpl() call.
     */
    protected function display( $viewModel )
    {
        $view = new View($viewModel);

        try {
            $view->display();
        }
        catch ( DAPluginException $e ) {
            $this->displayMissingTpl($e->getMessage());
        }
    }

}
