<?php

/** ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author    Michael Alegre
 * @copyright 2019-2023 LiteSpeed Technologies, Inc.
 * @since     2.0
 * ******************************************* */

namespace LsPanel;

/**
 * @since 2.0
 */
class DALiteSpeedUtil
{

    /**
     * @since 2.0
     * @var   null|DALiteSpeedUtil
     */
    private static $instance;

    /**
     * @since 2.1
     * @var   string
     */
    private static $lswsHome = '';

    /**
     * @since 2.1
     * @var   bool
     */
    private static $sharedCodeLoaded = false;

    /**
     *
     * @since 2.0
     */
    private function __construct() {}

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown when DALiteSpeedUtil has already been
     *     initialized.
     */
    public static function initialize()
    {
        if ( self::$instance != null ) {
            throw new DAPluginException(
                'DALiteSpeedUtil cannot be initialized twice.',
                DAPluginException::E_PROGRAM
            );
        }

        self::$instance = new static();
    }

    /**
     *
     * @since 2.1
     *
     * @return string
     */
    public static function getLswsHome()
    {
        if ( self::$lswsHome == '' ) {
            self::setLswsHome();
        }

        return self::$lswsHome;
    }

    /**
     *
     * @since 2.1
     *
     * @param bool $loaded
     */
    public static function setSharedCodeLoaded( $loaded )
    {
        self::$sharedCodeLoaded = $loaded;
    }

    /**
     *
     * @since 2.1
     *
     * @return bool
     */
    public static function sharedCodeLoaded()
    {
        return self::$sharedCodeLoaded;
    }

    /**
     *
     * @since 2.1
     *
     * @param string $lswsHome  Default value is the fixed location in DA.
     *
     * @return bool
     */
    public static function setLswsHome( $lswsHome = '/usr/local/lsws' )
    {
        if ( is_executable("$lswsHome/bin/lshttpd") ) {
            self::$lswsHome = $lswsHome;
            return true;
        }

        return false;
    }

    /**
     *
     * @since 2.0
     *
     * @param string $tag
     * @param mixed  $default  The default value to be passed back when the
     *     request var is not found.
     *
     * @return mixed|null|string
     */
    public static function get_request_var( $tag, $default = null )
    {
        if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
            $querystring = rawurldecode(getenv('POST'));
        }
        else {
            $querystring = rawurldecode(getenv('QUERY_STRING'));
        }

        if ( $querystring != ''
                && preg_match("/(?:^|\?|&)$tag=([^&]+)/", $querystring, $m) ) {

            if ( ($varValue = $m[1]) != null ) {
                return trim($varValue);
            }
        }

        return $default;
    }

    /**
     * Used as a wrapper function for exec() calls.
     *
     * @since 2.0
     *
     * @param string        $cmd
     * @param null|string[] $output
     * @param null|int      $ret
     */
    public static function exec_cmd(
        $cmd,
        &$output = null,
        &$ret = null )
    {
        exec($cmd, $output, $ret);
    }

    /**
     * Used as a wrapper for shell_exec() calls.
     *
     * @since 2.0
     *
     * @param string $cmd
     *
     * @return null|string
     */
    public static function shell_exec_cmd( $cmd )
    {
        return shell_exec($cmd);
    }

    /**
     * Returns LiteSpeed Web Server process ID, or 0 if it is not running.
     *
     * @since 2.0
     *
     * @return int
     */
    public static function getLSPID()
    {
        $pidFile = '/tmp/lshttpd/lshttpd.pid';

        if ( file_exists($pidFile) ) {

            if ( ($content = file_get_contents($pidFile)) === false
                    || ($pid = trim($content)) == '' ) {

                return 0;
            }

            $pid = (int)$pid;

            posix_kill($pid, 0);

            if ( posix_get_last_error() == 1 ) {
                /** EPERM */
                return $pid;
            }
        }

        return 0;
    }

    /**
     * Returns Apache process ID, or 0 if it is not running.
     *
     * @since 2.0
     *
     * @return int
     */
    public static function getApachePID()
    {
        $pidFile = '/var/run/httpd.pid';

        if ( file_exists($pidFile) ) {

            if ( ($content = file_get_contents($pidFile)) === false
                    || ($pid = trim($content)) == '' ) {

                return 0;
            }

            $pid = (int)$pid;

            posix_kill($pid, 0);

            if ( posix_get_last_error() == 1 ) {
                /** EPERM */
                return $pid;
            }
        }

        return 0;
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown indirectly by DAEscalate::escalate()
     *     call.
     */
    public static function restartLSWS()
    {
        DAEscalate::escalate(true, DAEscalate::LSWS_CMD_RESTART_LSWS);
    }

    /**
     *
     * @since 2.0
     *
     * @param string $file
     * @param int    $pid
     *
     * @return bool
     */
    public static function pidBackup( $file, $pid )
    {
        return rename($file, "$file.bak.$pid");
    }

    /**
     *
     * @since 2.0
     *
     * @param int    $pid
     * @param string $file
     */
    public static function undoPidBackup( $pid, $file )
    {
        rename("$file.bak.$pid", $file);
    }

    /**
     *
     * @since 2.0
     *
     * @return string[]
     *
     * @throws DAPluginException  Thrown indirectly by DAEscalate::escalate()
     *     call.
     */
    public static function getMainMenuInfo()
    {
        $info = DAEscalate::escalate(
            false,
            DAEscalate::LSWS_CMD_GET_MAINMENU_INFO,
            array( 'serv_type' => constant('LSWS_SERVER_TYPE') )
        );

        if ( !isset($info['serial']) ) {
            $info['serial'] = 'NO SERIAL FOUND';
        }

        if ( !isset($info['adminurl']) ) {
            $info['adminurl'] = '';
        }

        return $info;
    }

    /**
     *
     * @since 2.0
     *
     * @param string $output
     *
     * @return int
     *
     * @throws DAPluginException  Thrown indirectly by DAEscalate::escalate()
     *     call.
     */
    public static function getCurrentLicenseStatus( &$output )
    {
        $result = DAEscalate::escalate(
            true,
            DAEscalate::LSWS_CMD_GET_LICENSE_STATUS
        );

        if ( isset($result['status']) && $result['status'] != '' ) {
            $output .= "<p>{$result['status']}</p>\n";
        }

        if ( isset($result['MESG']) && $result['MESG'] != '' ) {
            $output .= "<p>{$result['MESG']}</p>\n";
        }

        return $result['return_var'];
    }

    /**
     *
     * @since 2.0
     *
     * @param string $serial_no
     *
     * @return string
     */
    public static function validate_serial_no( $serial_no )
    {
        if ( $serial_no == "" ) {
            return 'Missing serial number!';
        }
        else {
            $pattern = '[a-zA-Z0-9/+]{4}';

            $validSerialPattern = "%^$pattern-$pattern-$pattern-$pattern$%";

            if ( strlen($serial_no) != 19
                    || !preg_match($validSerialPattern, $serial_no) ) {

                return 'Invalid serial number';
            }
        }

        return '';
    }

    /**
     *
     * @since 2.0
     *
     * @param string[][] $info
     */
    public static function validateLicenseChangeInput( array &$info )
    {
        $errors = &$info['errors'];

        $info['license_agree'] = self::get_request_var('license_agree', '');

        if ( $info['license_agree'] != 'agree' ) {
            $errors['license_agree'] =
                'Cannot proceed without agreement to EULA';
        }

        $info['install_type'] = self::get_request_var('install_type', '');

        switch ( $info['install_type'] ) {

            case 'trial':
                $info['serial_no'] = 'TRIAL';
                break;

            case 'prod':
                $serial_no = trim(self::get_request_var('serial_no', ''));

                if ( ($err = self::validate_serial_no($serial_no)) != "" ) {
                    $errors['serial_no'] = $err;
                }
                else {
                    $info['serial_no'] = $serial_no;
                }

                break;

            case '':
                $errors['install_type'] = 'Please select one';
                break;

            default:
                $errors['install_type'] = 'Invalid type';
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param string[][] $info
     *
     * @return string
     *
     * @throws DAPluginException  Thrown indirectly by DAEscalate::escalate()
     *     call.
     */
    public static function switchLicense( array &$info )
    {
        $info['output'] = '';

        $result = DAEscalate::escalate(
            true,
            DAEscalate::LSWS_CMD_SWITCH_LICENSE,
            array ( 'serial_no' => $info['serial_no'] )
        );

        if ( isset($result['status']) ) {
            $info['output'] .= $result['status'];
        }

        if ( !empty($result['MESG']) ) {
            $info['output'] .= "<p>{$result['MESG']}</p>\n";
        }

        return $result['return_var'];
    }

    /**
     *
     * @since 2.0
     *
     * @param string[] $output
     *
     * @return string
     */
    public static function getOutputString( array $output )
    {
        $buf = '';

        foreach ( $output as $o ) {

            if ( $o != '' ) {
                $buf .= "$o <br />";
            }
        }

        return $buf;
    }

    /**
     *
     * @since 2.0
     *
     * @param string $url
     * @param bool   $headerOnly
     *
     * @return string
     */
    public static function getUrlContents( $url, $headerOnly = false )
    {
        if ( ini_get('allow_url_fopen') ) {
            /**
             * Silence warning when OpenSSL missing while getting LSCWP ver
             * file.
             */
            if ( ($url_content = @file_get_contents($url)) !== false ) {

                if ( $headerOnly ) {
                    return implode("\n", $http_response_header);
                }

                return $url_content;
            }
        }

        if ( function_exists('curl_version') ) {
            $ch = curl_init();

            curl_setopt_array(
                $ch,
                array(
                    CURLOPT_URL => $url,
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_HEADER => $headerOnly,
                    CURLOPT_NOBODY => $headerOnly,
                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1
                )
            );

            $url_content = curl_exec($ch);
            curl_close($ch);

            if ( $url_content !== false ) {
                return $url_content;
            }
        }

        $cmd = 'curl';

        if ( $headerOnly ) {
            $cmd .= ' -s -I';
        }

        exec("$cmd $url", $output, $ret);

        if ( $ret === 0 ) {
            return implode("\n", $output);
        }

        return '';
    }

    /**
     * Recursively remove a directory's contents and optionally the directory
     * itself. Intended to be used by DAEscalate only.
     *
     * @since 2.0
     *
     * @param string $dir         Directory path
     * @param bool   $keepParent  Only remove directory contents when true.
     *
     * @return bool
     */
    public static function e2_rrmdir( $dir, $keepParent = false )
    {
        if ( $dir != '' && is_dir($dir) ) {

            if ( ($matches = glob("$dir/*")) === false ) {
                return false;
            }

            foreach ( $matches as $file ) {

                if ( is_dir($file) ) {
                    self::e2_rrmdir($file);
                }
                else {
                    unlink($file);
                }
            }

            if ( !$keepParent ) {
                rmdir($dir);
            }

            return true;
        }

        return false;
    }

    /**
     * Version comparison function capable of properly comparing versions with
     * trailing ".0" groups such as '6.1' which is equal to '6.1.0' which is
     * equal to '6.1.000.0' etc.
     *
     * @since 2.3
     *
     * @param string      $ver1
     * @param string      $ver2
     * @param string|null $operator
     *
     * @return bool|int
     */
    public static function betterVersionCompare(
        $ver1,
        $ver2,
        $operator = null )
    {
        $trimPattern = '/(\.0+)+($|-)/';

        return version_compare(
            preg_replace($trimPattern, '', $ver1),
            preg_replace($trimPattern, '', $ver2),
            $operator
        );
    }

    /**
     *
     * @since 2.3
     *
     * @return string
     *
     * @throws DAPluginException  Thrown indirectly by DAEscalate::escalate()
     *     call.
     */
    public static function newUpdateSharedCode()
    {
        return DAEscalate::escalate(
            true,
            DAEscalate::MGR_CMD_UPDATE_SHARED_CODE
        );
    }

    /**
     *
     * @since 2.3.1
     *
     * @param string                           $constantName
     * @param array|bool|float|int|null|string $value
     * @param bool                             $caseInsensitive  Optional
     *     parameter used for define calls in PHP versions below 7.3.
     *
     * @return bool
     *
     * @noinspection PhpDeprecationInspection  Ignore deprecation of define()
     *     parameter $case_insensitive for PHP versions below 7.3.
     * @noinspection RedundantSuppression
     */
    public static function define_wrapper(
        $constantName,
        $value,
        $caseInsensitive = false )
    {
        if ( PHP_VERSION_ID < 70300 ) {
            return define($constantName, $value, $caseInsensitive);
        }
        else {
            return define($constantName, $value);
        }
    }

    /**
     * The following functions have been deprecated.
     */

    /**
     *
     * @deprecated 2.1
     *
     * @since 2.0
     *
     * @param string $dir
     *
     * @return false|string
     */
    public static function directoryMd5( $dir )
    {
        if ( !is_dir($dir) ) {
            return false;
        }

        $filemd5s = array();
        $d        = dir($dir);

        while ( ($entry = $d->read()) !== false ) {

            if ( $entry != '.' && $entry != '..' ) {

                $currEntry = "$dir/$entry";

                if ( is_dir($currEntry) ) {
                    $filemd5s[] = self::directoryMd5($currEntry);
                }
                else {
                    $filemd5s[] = md5_file($currEntry);
                }
            }
        }

        $d->close();
        return md5(implode('', $filemd5s));
    }

    /**
     *
     * @deprecated 2.3  Use DALiteSpeedUtil::newUpdateSharedCode() instead.
     *
     * @since 2.0
     *
     * @return bool
     *
     * @throws DAPluginException  Thrown indirectly by
     *     self::newUpdateSharedCode() call.
     */
    public static function updateSharedCode()
    {
        return (self::newUpdateSharedCode() == '');
    }
}
