<?php

/** ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author Michael Alegre
 * @copyright (c) 2019-2023 LiteSpeed Technologies, Inc.
 * @since 2.0
 * ******************************************* */

namespace LsPanel;

use Lsc\Wp\LSCMException;
use Lsc\Wp\Panel\ControlPanel;
use Lsc\Wp\PanelController;
use Lsc\Wp\PluginVersion;
use Lsc\Wp\View\Model as LscViewModel;
use LsPanel\Extend\DAPanelController;
use LsPanel\Extend\DirectAdminPanel;
use LsPanel\View\Model as DAViewModel;

/**
 * @since 2.0
 */
class DAOpenLiteSpeedController extends DALiteSpeedControllerBase
{

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown indirectly by parent::__construct().
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     *
     * @since 2.3
     *
     * @param null|string $do
     *
     * @return bool
     */
    private static function isSupportedAction( $do )
    {
        return in_array(
            $do,
            array (
                'restart_lsws',
                'lscwp_manage',
                'lscwp_mass_enable_disable',
                'lscwpVersionManager',
                'dash_notifier',
                'dash_slug_verify',
                'updateSharedCode',
                'main',
                ''
            )
        );
    }

    /**
     *
     * @since 2.0
     * @since 2.1  Added optional parameters $do, $step, $mgrStep, and
     *     $dashStep.
     *
     * @param null|string $do
     * @param null|int    $step
     * @param null|int    $mgrStep
     * @param null|int    $dashStep
     *
     * @throws DAPluginException  Thrown when $do, $step, $mgrStep, or $dashStep
     *     values ore unrecognized.
     * @throws DAPluginException  Thrown indirectly by $this->restartLsws()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->doSharedCodeDependentAction() call.
     * @throws DAPluginException  Thrown indirectly by $this->run() call.
     * @throws DAPluginException  Thrown indirectly by $this->dashSlugVerify()
     *     call.
     * @throws DAPluginException  Thrown indirectly by $this->updateSharedCode()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->getMainMenuViewModel() call.
     * @throws DAPluginException  Thrown indirectly by $this->display() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->handleDefaultAjaxResponse() call.
     */
    public function run(
        $do = null,
        $step = null,
        $mgrStep = null,
        $dashStep = null )
    {
        parent::setRunRequestVars($do, $step, $mgrStep, $dashStep);

        if ( !self::isSupportedAction($do)
                || !in_array($step, parent::RANGE_STEP)
                || !in_array($mgrStep, parent::RANGE_MGR_STEP)
                || !in_array($dashStep, parent::RANGE_DASH_STEP) ) {

            throw new DAPluginException('Invalid Entrance');
        }

        ob_start();

        switch ($do) {

            case 'restart_lsws':
                $viewModel = $this->restartLsws($step);
                break;

            case 'lscwp_manage':
            case 'lscwp_mass_enable_disable':
            case 'lscwpVersionManager':
            case 'dash_notifier':
                $ret = $this->doSharedCodeDependentAction(
                    $do,
                    $step,
                    $mgrStep,
                    $dashStep
                );

                if ( $ret == self::REDIRECT_TO_MAIN_MENU ) {
                    $this->run('main', 0, 0, 0);
                }

                $viewModel = $ret;
                break;

            case 'dash_slug_verify':
                $this->dashSlugVerify();
                break;

            case 'updateSharedCode':
                $viewModel = $this->updateSharedCode();
                break;

            case 'main':
            case '':
                $viewModel = $this->getMainMenuViewModel();
                break;

            //no default
        }

        if ( !empty($viewModel) ) {
            $this->display($viewModel);
        }

        if ( defined('LSWS_AJAX_CALL') && LSWS_AJAX_CALL ) {
            $this->handleDefaultAjaxResponse(ob_get_clean());
        }

        ob_flush();
    }

    /**
     *
     * @since 2.0
     *
     * @param string $do
     * @param int    $step
     * @param int    $mgrStep
     * @param int    $dashStep
     *
     * @return object|string|null  ViewModel object, a redirection string, or
     *     null if called with an improper $do value.
     *
     * @throws DAPluginException  Thrown indirectly by $this->lscwpManage()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->lscwpMassEnableDisableControl() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->lscwpVersionManager() call.
     * @throws DAPluginException  Thrown indirectly by $this->dashNotifier()
     *     call.
     */
    private function doSharedCodeDependentAction(
        $do,
        $step,
        $mgrStep,
        $dashStep )
    {
        if ( ! DALiteSpeedUtil::sharedCodeLoaded()
                || DALiteSpeedUtil::betterVersionCompare(ControlPanel::PANEL_API_VERSION, '1.9.7', '<')
                || ! DirectAdminPanel::meetsMinAPIVerRequirement() ) {

            return (new DAViewModel\SharedCodeUpdateRequiredViewModel());
        }

        $panelAPICompatStatus =
            DirectAdminPanel::checkPanelAPICompatibility(self::SHARED_API_VER);

        switch ( $panelAPICompatStatus ) {

            case ControlPanel::PANEL_API_VERSION_TOO_HIGH:
                return (new DAViewModel\SharedCodeUpdateRequiredViewModel());

            case ControlPanel::PANEL_API_VERSION_TOO_LOW:
            case ControlPanel::PANEL_API_VERSION_UNKNOWN:
                return (new DAViewModel\PluginUpdateRequiredViewModel());

            //no default
        }

        switch ($do) {

            case 'lscwp_manage':
                return $this->lscwpManage($mgrStep);

            case 'lscwp_mass_enable_disable':
                return $this->lscwpMassEnableDisableControl($step);

            case 'lscwpVersionManager':
                return $this->lscwpVersionManager($step);

            case 'dash_notifier':
                return $this->dashNotifier($dashStep);

            default:
                return null;
        }
    }

    /**
     *
     * @since 2.0
     *
     * @return DAViewModel\Open_MainMenuViewModel
     *
     * @throws DAPluginException  Thrown indirectly by
     *     "new DAViewModel\Open_MainMenuViewModel()" call.
     *
     * @noinspection PhpDocRedundantThrowsInspection
     */
    protected function getMainMenuViewModel()
    {
        /**
         * Throws DAPluginException.
         */
        return (new DAViewModel\Open_MainMenuViewModel());
    }

    /**
     * Note: Assume that cache roots are set for OLS in DA env.
     *
     * @since 2.0
     *
     * @param int $mgrStep
     *
     * @return object  ViewModel object.
     *
     * @throws DAPluginException  (Re-)Thrown when calls
     *     PluginVersion::getInstance() or
     *     $panelController->manageCacheOperations2() throw an LSCMException.
     * @throws DAPluginException  (Re-)Thrown when call
     *     "new LscViewModel\ScanProgressViewModel()" throws an LSCMException.
     * @throws DAPluginException  (Re-)Thrown when call
     *     "new LscViewModel\ManageViewModel()" throws an LSCMException.
     * @throws DAPluginException  Thrown indirectly by
     *     self::getDAPanelWPInstallStorageInstance() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->getControlPanelInstance() call.
     */
    protected function lscwpManage( $mgrStep )
    {
        $model = self::getDAPanelWPInstallStorageInstance();

        $panelController = new DAPanelController(
            $this->getControlPanelInstance(),
            $model,
            $mgrStep
        );

        try {
            /**
             * Pre-initialize DAPanelPluginVersion instance for viewModel.
             */
            PluginVersion::getInstance();

            $newMgrStep = $panelController->manageCacheOperations2();
        }
        catch ( LSCMException $e ) {
            throw new DAPluginException($e->getMessage(), $e->getCode());
        }

        switch ($newMgrStep) {

            case PanelController::MGR_STEP_SCAN:
            case PanelController::MGR_STEP_DISCOVER_NEW:
                try {
                    return (
                        new LscViewModel\ScanProgressStepViewModel($newMgrStep)
                    );
                }
                catch ( LSCMException $e ) {
                    throw new DAPluginException(
                        $e->getMessage(),
                        $e->getCode()
                    );
                }

            case PanelController::MGR_STEP_REFRESH_STATUS:
                return (new LscViewModel\RefreshStatusProgressViewModel());

            case PanelController::MGR_STEP_MASS_UNFLAG:
                return (new LscViewModel\UnflagAllProgressViewModel());

            default:
                try {
                    return (new LscViewModel\ManageViewModel($model));
                }
                catch ( LSCMException $e ) {
                    throw new DAPluginException(
                        $e->getMessage(),
                        $e->getCode()
                    );
                }
        }
    }

    /**
     * Note: Assume that cache roots are set for OLS in DA env.
     *
     * @since 2.0
     *
     * @param int $step
     *
     * @return object|string  ViewModel object or redirection string.
     *
     * @throws DAPluginException  Thrown indirectly by
     *     self::getDAPanelWPInstallStorageInstance() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->lscwpMassEnableDisable() call.
     */
    protected function lscwpMassEnableDisableControl( $step )
    {
        $model = self::getDAPanelWPInstallStorageInstance();

        if ( $step == 1 || $step == 2 ) {
            $action = DALiteSpeedUtil::get_request_var('act');

            if ( in_array($action, array( 'enable', 'disable' )) ) {
                return $this->lscwpMassEnableDisable($action, $step, $model);
            }
        }
        elseif ( $step == 0 && $model->getError() != 0 ) {
            return (new LscViewModel\DataFileMsgViewModel($model));
        }

        return (new LscViewModel\MassEnableDisableViewModel($model));
    }

}
