<?php

/** ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author Michael Alegre
 * @copyright (c) 2019-2022 LiteSpeed Technologies, Inc.
 * @since 2.0
 * ******************************************* */

namespace LsPanel;

/**
 * DAPluginEscalatedLogger is a singleton.
 *
 * Functions starting with 'p_' are implemented for compatibility with shared
 * code Logger class and are not intended for use directly.
 *
 * @since 2.0
 */
class DAPluginEscalatedLogger
{

    /**
     * @since 2.0
     * @var string
     */
    const TEMP_ESCALATED_LOG_BASE = '/tmp/ls-da-esc-tmp';

    /**
     * @since 2.1
     * @var int
     */
    const L_NONE = 0;

    /**
     * @since 2.1
     * @var int
     */
    const L_ERROR = 1;

    /**
     * @since 2.1
     * @var int
     */
    const L_WARN = 2;

    /**
     * @since 2.1
     * @var int
     */
    const L_NOTICE = 3;

    /**
     * @since 2.1
     * @var int
     */
    const L_INFO = 4;

    /**
     * @since 2.1
     * @var int
     */
    const L_VERBOSE = 5;

    /**
     * @since 2.1
     * @var int
     */
    const L_DEBUG = 9;

    /**
     * @since 2.1
     * @var int
     */
    const UI_INFO = 0;

    /**
     * @since 2.1
     * @var int
     */
    const UI_SUCC = 1;

    /**
     * @since 2.1
     * @var int
     */
    const UI_ERR = 2;

    /**
     * @since 2.1
     * @var int
     */
    const UI_WARN = 3;

    /**
     * @since 2.1
     * @var null|DAPluginEscalatedLogger
     */
    private static $instance;

    /**
     * @since 2.1
     * @var int  Highest log message level allowed to be logged. Set to the
     *     higher value between $this->logFileLvl and $this->logEchoLvl.
     */
    private $logLvl;

    /**
     * @since 2.1
     * @var string  File that log messages will be written to (if writable).
     */
    private $logFile;

    /**
     * @since 2.1
     * @var int  Highest log message level allowed to be written to the log
     *     file.
     */
    private $logFileLvl;

    /**
     * @since 2.1
     * @var string  Additional tag to be added at the start of any log messages.
     */
    private $addTagInfo = '';

    /**
     * @since 2.1
     * @var boolean  When set to true, log messages will not be written to the
     *     log file immediately. Messages will be automatically written when
     *     this DAPluginEscalatedLogger object is destroyed.
     */
    private $bufferedWrite;

    /**
     * @since 2.1
     * @var DAPluginEscalatedLogEntry[]  Stores created
     *     DAPluginEscalatedLogEntry objects.
     */
    private $msgQueue = array();

    /**
     * @since 2.1
     * @var int  Highest log message level allowed to echoed.
     */
    private $logEchoLvl;

    /**
     * @since 2.1
     * @var boolean  When set to true, echoing of log messages will not occur
     *     immediately. Messages will be automatically echo'd when this
     *     DAPluginEscalatedLogger object is destroyed.
     */
    private $bufferedEcho;

    /**
     * @since 2.1
     * @var string[][]  Leveraged by Control Panel GUI to store and retrieve
     *     display messages.
     */
    private $uiMsgs = array(
        self::UI_INFO=> array(),
        self::UI_SUCC => array(),
        self::UI_ERR => array(),
        self::UI_WARN => array()
    );

    /**
     *
     * @since 2.0
     * @deprecated 2.1  Function visibility will be changed to private in the
     *     future.
     *
     * @throws DAPluginException  Thrown indirectly by $this->init() call.
     */
    protected function __construct()
    {
        $this->init();
    }

    /**
     *
     * @since 2.1
     *
     * @throws DAPluginException  Thrown when getmypid() call returns boolean
     *     false.
     */
    private function init()
    {
        if ( ($pid = getmypid()) === false ) {
            throw new DAPluginException(
                'Failed to get current PHP process ID when attempting to '
                    . 'initialize class DAPluginEscalatedLogger.'
            );
        }

        date_default_timezone_set('UTC');

        $this->logFile = self::TEMP_ESCALATED_LOG_BASE . "_$pid.log";

        $this->logFileLvl = self::L_INFO;
        $this->logEchoLvl = self::L_NONE;
        $this->bufferedWrite = true;
        $this->bufferedEcho = true;

        $this->logLvl = max($this->logEchoLvl, $this->logFileLvl);
    }

    /**
     *
     * @since 2.0
     *
     * @param int $parentPid
     *
     * @throws DAPluginException  Thrown indirectly by self::initialize() call.
     */
    public static function InitializeWithParentPid( $parentPid )
    {
        self::initialize();

        self::$instance->logFile =
            self::TEMP_ESCALATED_LOG_BASE . "_$parentPid.log";
    }

    /**
     *
     * @since 2.1
     *
     * @throws DAPluginException  Thrown indirectly by $this->writeToFile()
     *     call.
     */
    public function __destruct()
    {
        if ( $this->bufferedWrite ) {
            $this->writeToFile($this->msgQueue);
        }
    }

    /**
     *
     * @since 2.1
     *
     * @throws DAPluginException  Thrown when class DAPluginEscalatedLogger
     *     class has already been initialized.
     * @throws DAPluginException  Thrown indirectly by "new static()" call.
     */
    public static function initialize()
    {
        if ( self::$instance != null ) {
            throw new DAPluginException(
                'DAPluginEscalatedLogger class cannot be initialized twice.',
                DAPluginException::E_PROGRAM
            );
        }

        self::$instance = new static();
    }

    /**
     *
     * @since 2.1
     *
     * @return null|DAPluginEscalatedLogger
     */
    public static function getInstance()
    {
        return self::$instance;
    }

    /**
     * Create log file if it does not already exist. Useful for creating temp
     * log file with correct permissions before writing to it in escalated
     * functions.
     *
     * @since 2.1
     *
     * @throws DAPluginException  Thrown indirectly.
     */
    public static function preCreateLogFile()
    {
        if ( !file_exists(self::me()->logFile) ) {
            touch(self::me()->logFile);
        }
    }

    /**
     *
     * @since 2.1
     *
     * @param string $logFile
     *
     * @throws DAPluginException  Thrown indirectly by self::me() call.
     */
    public static function changeLogFileUsed( $logFile )
    {
        self::me()->logFile = $logFile;
    }

    /**
     *
     * @since 2.1
     *
     * @param string $addInfo
     *
     * @throws DAPluginException  Thrown indirectly by self::me() call.
     */
    public static function setAdditionalTagInfo( $addInfo )
    {
        self::me()->addTagInfo = $addInfo;
    }

    /**
     *
     * @since 2.1
     *
     * @param string $msg
     * @param int    $type
     */
    public function p_addUiMsg( $msg, $type )
    {
        switch ($type) {
            case self::UI_INFO:
            case self::UI_SUCC:
            case self::UI_ERR:
            case self::UI_WARN:
                $this->uiMsgs[$type][] = $msg;
                break;

            //no default
        }
    }

    /**
     *
     * @since 2.1
     *
     * @param DAPluginEscalatedLogEntry[] $entries
     *
     * @throws DAPluginException  Thrown indirectly by $this->echoEntries()
     *     call.
     */
    public function p_echoEntries( array $entries )
    {
        $this->echoEntries($entries);
    }

    /**
     *
     * @since 2.1
     *
     * @return string
     */
    public function p_getAddTagInfo()
    {
        return $this->addTagInfo;
    }

    /**
     *
     * @since 2.1
     *
     * @return bool
     */
    public function p_getBufferedEcho()
    {
        return $this->bufferedEcho;
    }

    /**
     *
     * @since 2.1
     *
     * @return bool
     */
    public function p_getBufferedWrite()
    {
        return $this->bufferedWrite;
    }

    /**
     *
     * @since 2.1
     *
     * @return string
     */
    public function p_getLogFile()
    {
        return $this->logFile;
    }

    /**
     *
     * @since 1.3
     *
     * @return DAPluginEscalatedLogEntry[]
     */
    public function p_getMsgQueue()
    {
        return $this->msgQueue;
    }

    /**
     *
     * @since 2.1
     *
     * @param int $type
     *
     * @return string[]
     */
    public function p_getUiMsgs( $type )
    {
        $ret = array();

        switch ($type) {
            case self::UI_INFO:
            case self::UI_SUCC:
            case self::UI_ERR:
            case self::UI_WARN:
                $ret = $this->uiMsgs[$type];
                break;

            //no default
        }

        return $ret;
    }

    /**
     *
     * @since 2.1
     *
     * @param string $msg
     * @param int    $lvl
     *
     * @throws DAPluginException  Thrown indirectly by $this->log() call.
     */
    public function p_log( $msg, $lvl )
    {
        $this->log($msg, $lvl);
    }

    /**
     *
     * @since 2.1
     *
     * @param string $addInfo
     */
    public function p_setAddTagInfo( $addInfo )
    {
        $this->addTagInfo = $addInfo;
    }

    /**
     *
     * @since 2.1
     *
     * @param string $logFile
     */
    public function p_setLogFile( $logFile )
    {
        $this->logFile = $logFile;
    }

    /**
     *
     * @since 2.1
     *
     * @param int $logFileLvl
     */
    public function p_setLogFileLvl( $logFileLvl )
    {
        $this->logFileLvl = $logFileLvl;
    }

    /**
     *
     * @since 2.1
     *
     * @param DAPluginEscalatedLogEntry[] $msgQueue
     */
    public function p_setMsgQueue( array $msgQueue )
    {
        $this->msgQueue = $msgQueue;
    }

    /**
     *
     * @since 2.1
     *
     *
     * @param DAPluginEscalatedLogEntry[] $entries  Array of objects that
     *     implement all LogEntry class public functions.
     *
     * @throws DAPluginException  Thrown indirectly by $this->>writeToFile()
     *     call.
     */
    public function p_writeToFile( $entries )
    {
        $this->writeToFile($entries);
    }

    /**
     *
     * @since 2.1
     *
     * @return string
     *
     * @throws DAPluginException  Thrown indirectly by self::me() call.
     */
    public static function getAdditionalTagInfo()
    {
        return self::me()->addTagInfo;
    }

    /**
     *
     * @since 2.1
     *
     * @return string
     *
     * @throws DAPluginException  Thrown indirectly by self::me() call.
     */
    public static function getLogFilePath()
    {
        return self::me()->logFile;
    }

    /**
     *
     * @since 2.1
     *
     * @return DAPluginEscalatedLogEntry[] Array of objects that implement all
     *     LogEntry class public functions.
     *
     * @throws DAPluginException  Thrown indirectly by self::me() call.
     */
    public static function getLogMsgQueue()
    {
        return self::me()->p_getMsgQueue();
    }

    /**
     *
     * @since 2.1
     *
     * @param int $type
     *
     * @return string[]
     *
     * @throws DAPluginException  Thrown indirectly by self::me() call.
     */
    public static function getUiMsgs( $type )
    {
        return self::me()->p_getUiMsgs($type);
    }

    /**
     * Processes any buffered output, writing it to the log file, echoing it
     * out, or both.
     *
     * @since 2.1
     *
     * @throws DAPluginException  Thrown indirectly by self::me() call.
     * @throws DAPluginException  Thrown indirectly by $m->writeToFile() call.
     * @throws DAPluginException  Thrown indirectly by $m->echoEntries() call.
     */
    public static function processBuffer()
    {
        $clear = false;

        $m = self::me();

        if ( $m->bufferedWrite ) {
            $m->writeToFile($m->msgQueue);
            $clear = true;
        }

        if ( $m->bufferedEcho ) {
            $m->echoEntries($m->msgQueue);
            $clear = true;
        }

        if ( $clear ) {
            $m->msgQueue = array();
        }
    }

    /**
     * This deprecated function exists to more closely match the public API of
     * the shared code Logger class in which an object of this class may be
     * used as the value of the Logger class' protected $instance class
     * variable.
     *
     * @deprecated 2.1
     *
     * @since 2.1
     *
     * @param string $msg
     * @param int    $type
     *
     * @throws DAPluginException  Thrown indirectly by self::me() call.
     */
    public static function addUiMsg( $msg, $type )
    {
        self::me()->p_addUiMsg($msg, $type);
    }

    /**
     * Calls addUiMsg() with message level self::UI_INFO.
     *
     * @since 2.1
     *
     * @param string $msg
     *
     * @throws DAPluginException  Thrown indirectly by self::addUiMsg() call.
     */
    public static function uiInfo( $msg )
    {
        self::addUiMsg($msg, self::UI_INFO);
    }

    /**
     * Calls addUiMsg() with message level self::UI_SUCC.
     *
     * @since 2.1
     *
     * @param string $msg
     *
     * @throws DAPluginException  Thrown indirectly by self::addUiMsg() call.
     */
    public static function uiSuccess( $msg )
    {
        self::addUiMsg($msg, self::UI_SUCC);
    }

    /**
     * Calls addUiMsg() with message level self::UI_ERR.
     *
     * @since 2.1
     *
     * @param string $msg
     *
     * @throws DAPluginException  Thrown indirectly by self::addUiMsg() call.
     */
    public static function uiError( $msg )
    {
        self::addUiMsg($msg, self::UI_ERR);
    }

    /**
     * Calls addUiMsg() with message level self::UI_WARN.
     *
     * @since 2.1
     *
     * @param string $msg
     *
     * @throws DAPluginException  Thrown indirectly by self::addUiMsg() call.
     */
    public static function uiWarning( $msg )
    {
        self::addUiMsg($msg, self::UI_WARN);
    }

    /**
     *
     * @since 2.1
     *
     * @param string $msg
     * @param int    $lvl
     *
     * @throws DAPluginException  Thrown indirectly by self::me() call.
     * @throws DAPluginException  Thrown indirectly by self::me()->log() call.
     */
    private static function logMsg( $msg, $lvl )
    {
        self::me()->log($msg, $lvl);
    }

    /**
     * Calls logMsg() with message level self::L_ERROR.
     *
     * @since 2.1
     *
     * @param string $msg
     *
     * @throws DAPluginException  Thrown indirectly by self::logMsg() call.
     */
    public static function error( $msg )
    {
        self::logMsg($msg, self::L_ERROR);
    }

    /**
     * Calls logMsg() with message level self::L_WARN.
     *
     * @since 2.1
     *
     * @param string $msg
     *
     * @throws DAPluginException  Thrown indirectly by self::logMsg() call.
     */
    public static function warn( $msg )
    {
        self::logMsg($msg, self::L_WARN);
    }

    /**
     * Calls logMsg() with message level self::L_NOTICE.
     *
     * @since 2.1
     *
     * @param string $msg
     *
     * @throws DAPluginException  Thrown indirectly by self::logMsg() call.
     */
    public static function notice( $msg )
    {
        self::logMsg($msg, self::L_NOTICE);
    }

    /**
     * Calls logMsg() with message level self::L_INFO.
     *
     * @since 2.1
     *
     * @param string $msg
     *
     * @throws DAPluginException  Thrown indirectly by self::logMsg() call.
     */
    public static function info( $msg )
    {
        self::logMsg($msg, self::L_INFO);
    }

    /**
     * Calls logMsg() with message level self::L_VERBOSE.
     *
     * @since 2.1
     *
     * @param string $msg
     *
     * @throws DAPluginException  Thrown indirectly by self::logMsg() call.
     */
    public static function verbose( $msg )
    {
        self::logMsg($msg, self::L_VERBOSE);
    }

    /**
     * Calls logMsg() with message level self::L_DEBUG.
     *
     * @since 2.1
     *
     * @param string $msg
     *
     * @throws DAPluginException  Thrown indirectly by self::logMsg() call.
     */
    public static function debug( $msg )
    {
        self::logMsg($msg, self::L_DEBUG);
    }

    /**
     *
     * @since 2.1
     *
     * @return DAPluginEscalatedLogger
     *
     * @throws DAPluginException  Thrown when DAPluginLogger class is
     *     uninitialized.
     */
    private static function me()
    {
        if ( self::$instance == null ) {
            throw new DAPluginException(
                'Plugin logger class uninitialized.',
                DAPluginException::E_PROGRAM
            );
        }

        return self::$instance;
    }

    /**
     *
     * @since 2.1
     *
     * @param string $mesg
     * @param int    $lvl
     *
     * @throws DAPluginException  Thrown indirectly by $this->writeToFile()
     *     call.
     * @throws DAPluginException  Thrown indirectly by $this->echoEntries()
     *     call.
     */
    protected function log( $mesg, $lvl )
    {
        $entry = new DAPluginEscalatedLogEntry($mesg, $lvl);

        $this->msgQueue[] = $entry;

        if ( !$this->bufferedWrite ) {
            $this->writeToFile(array( $entry ));
        }

        if ( !$this->bufferedEcho ) {
            $this->echoEntries(array( $entry ));
        }
    }

    /**
     * Write temp escalated log file contents.
     *
     * @since 2.0
     *
     * @param DAPluginEscalatedLogEntry[] $entries
     *
     * @throws DAPluginException  Thrown indirectly by $e->getOutput() call.
     */
    protected function writeToFile( array $entries )
    {
        $content = '';

        foreach ( $entries as $e ) {
            $content .= $e->getOutput($this->logFileLvl);
        }

        if ( $content != '' ) {

            if ( $this->logFile ) {
                file_put_contents(
                    $this->logFile,
                    $content,
                    FILE_APPEND | LOCK_EX
                );
            }
            else {
                error_log($content);
            }
        }
    }

    /**
     *
     * @since 2.1
     *
     * @param DAPluginEscalatedLogEntry[] $entries
     *
     * @throws DAPluginException  Thrown indirectly by $entry->getOutput() call.
     */
    private function echoEntries( array $entries )
    {
        foreach ( $entries as $entry ) {

            if ( ($msg = $entry->getOutput($this->logEchoLvl)) !== '' ) {
                echo $msg;
            }
        }
    }

    /**
     *
     * @since 2.1
     *
     * @param int $lvl
     *
     * @return string
     */
    public static function getLvlDescr( $lvl )
    {
        switch ($lvl) {

            case self::L_ERROR:
                return 'ERROR';

            case self::L_WARN:
                return 'WARN';

            case self::L_NOTICE:
                return 'NOTICE';

            case self::L_INFO:
                return 'INFO';

            case self::L_VERBOSE:
                return 'DETAIL';

            case self::L_DEBUG:
                return 'DEBUG';

            default:
                /**
                 * Do silently.
                 */
                return '';
        }
    }

    /**
     * This deprecated function exists to more closely match the public API of
     * the shared code Logger class in which an object of this class may be
     * used as the value of the Logger class' protected $instance class
     * variable.
     *
     * @deprecated 2.1
     *
     * @since 2.1
     *
     * @param int $lvl
     *
     * @return bool
     *
     * @throws DAPluginException  Thrown indirectly by self::me() call.
     */
    public static function setLogFileLvl( $lvl )
    {
        $lvl = (int)$lvl;

        if ( self::isValidLogFileLvl($lvl) ) {

            if ( $lvl > self::L_DEBUG ) {
                $lvl = self::L_DEBUG;
            }

            self::me()->p_setLogFileLvl($lvl);

            return true;
        }

        return false;
    }

    /**
     *
     * @since 2.1
     *
     * @param int $lvl
     *
     * @return bool
     */
    private static function isValidLogFileLvl( $lvl )
    {
        if ( is_int($lvl) && $lvl >= 0 ) {
            return true;
        }

        return false;
    }

    /**
     * Prevent cloning here and in extending classes.
     *
     * @since 2.1
     */
    private function __clone() {}

}