<?php

/** ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019-2023
 * @since 2.0
 * ******************************************* */

namespace LsPanel\Extend;

use \Lsc\Wp\AjaxResponse;
use \Lsc\Wp\Logger;
use \Lsc\Wp\LSCMException;
use \Lsc\Wp\Panel\ControlPanel;
use \Lsc\Wp\PanelController;
use \Lsc\Wp\View\Model\Ajax\CacheMgrRowViewModel;
use \Lsc\Wp\WPInstallStorage;
use LsPanel\DAEscalate;
use LsPanel\DAPluginException;

/**
 * @since 2.0
 */
class DAPanelController extends PanelController
{

    /**
     *
     * @since 2.0
     *
     * @param ControlPanel      $panelEnv
     * @param WPInstallStorage  $wpInstallStorage
     * @param int               $mgrStep
     */
    public function __construct( ControlPanel $panelEnv,
            WPInstallStorage $wpInstallStorage, $mgrStep = self::MGR_STEP_NONE )
    {
        /**
         * pre-set ajax reponse header
         */
        AjaxResponse::setHeaderContent(
            "HTTP/1.1 200 OK\nContent-Type: application/json"
        );

        parent::__construct($panelEnv, $wpInstallStorage, $mgrStep);
    }

    /**
     *
     * @since 2.3
     *
     * @param string[] $docroots
     * @param string[] $wpPaths  Array of discovered WordPress installations
     *     (passed by address).
     *
     * @return int  Completed count.
     *
     * @throws LSCMException  Rethrown when DAEscalate::escalate() call throws a
     *     DAPluginException exception.
     */
    protected function runScan2Logic( array $docroots, array &$wpPaths )
    {
        try {
            $ret = DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_SCAN_2_BATCH,
                array( 'docroots' => $docroots )
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }

        $wpPaths = array_merge($ret['wpPaths'], $wpPaths);

        return count($ret['finishedList']);
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $action
     * @param string  $path
     * @return void
     * @throws LSCMException  Thrown indirectly.
     */
    protected function doSingleAction( $action, $path )
    {
        $wpInstall = $this->wpInstallStorage->getWPInstall($path);

        if ( $wpInstall === null ) {
            Logger::uiError('Invalid input value detected - No Action Taken');
            return;
        }

        $badgesDown = $badgesUp = array();
        $removed = false;

        $preStatusInfo = $this->getStatusStrings($wpInstall);

        $this->wpInstallStorage->doAction($action, array( $path ));

        $wpInstall = $this->wpInstallStorage->getWPInstall($path);

        $postStatusInfo = $this->getStatusStrings($wpInstall);

        if ( $postStatusInfo['status'] == 'removed' ) {
            $removed = true;

            $badgesDown[] = $preStatusInfo['status'];
            $badgesDown[] = 'flag';
        }
        else {

            if ( $preStatusInfo['status'] != $postStatusInfo['status'] ) {
                $badgesDown[] = $preStatusInfo['status'];
                $badgesUp[] = $postStatusInfo['status'];
            }

            if ( $preStatusInfo['flag_status'] != $postStatusInfo['flag_status'] ) {

                if ( $postStatusInfo['flag_status'] == 'flagged' ) {
                    $badgesUp[] = 'flag';
                }
                else {
                    $badgesDown[] = 'flag';
                }
            }
        }

        $viewModel = new CacheMgrRowViewModel($wpInstall);
        $statusSort = $viewModel->getsortVal('status');
        $flagSort = $viewModel->getsortVal('flag');

        switch ( $flagSort ) {
            case 'removed':
                $flagSearch = $flagSort;
                break;
            case 'flagged':
                $flagSearch = 'f';
                break;
            default:
                $flagSearch = 'u';
        }

        $actions_td = $this->getAjaxViewContent($viewModel, 'actions_td');
        $status_td = $this->getAjaxViewContent($viewModel, 'status_td');
        $flag_td = $this->getAjaxViewContent($viewModel, 'flag_td');

        $msgs = $this->wpInstallStorage->getAllCmdMsgs();
        $errMsgs = array_merge($msgs['fail'], $msgs['err'],
                Logger::getUiMsgs(Logger::UI_ERR));
        $succMsgs = array_merge($msgs['succ'],
                Logger::getUiMsgs(Logger::UI_SUCC));

        $ajaxReturn = array(
            'displayData' => array(
                'actions_td' => $actions_td,
                'status_search' => $statusSort,
                'status_sort' => $statusSort,
                'status_td' => $status_td,
                'flag_search' => $flagSearch,
                'flag_sort' => $flagSort,
                'flag_td' => $flag_td
            ),
            'badgesDown' => $badgesDown,
            'badgesUp' => $badgesUp,
            'removed' => $removed,
            'errMsgs' => $errMsgs,
            'succMsgs' => $succMsgs
        );

        AjaxResponse::setAjaxContent(json_encode($ajaxReturn));
        AjaxResponse::outputAndExit();
    }

}
