<?php

/** ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author    Michael Alegre
 * @copyright 2019-2023 LiteSpeed Technologies, Inc.
 * @since     2.0
 * ******************************************* */

namespace LsPanel\Extend;

use Lsc\Wp\Logger;
use Lsc\Wp\LSCMException;
use Lsc\Wp\WPInstall;
use Lsc\Wp\WPInstallStorage;
use LsPanel\DAEscalate;
use LsPanel\DAPluginException;

/**
 * Map to data file
 *
 * @since 2.0
 */
class DAPanelWPInstallStorage extends WPInstallStorage
{

    /**
     *
     * @since 2.0
     *
     * @param string $dataFile
     * @param string $custDataFile
     *
     * @throws LSCMException  Thrown indirectly by parent::__construct() call.
     */
    public function __construct( $dataFile, $custDataFile = '' )
    {
        parent::__construct($dataFile, $custDataFile);
    }

    /**
     *
     * @since 2.0
     *
     * @return int
     *
     * @throws LSCMException  Thrown indirectly by Logger::debug() call.
     * @throws LSCMException  Thrown indirectly by Logger::debug() call.
     */
    protected function init()
    {
        $dataExists = false;

        try {
            $dataFileExists = DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_DATA_FILE_EXISTS,
                array( 'filePath' => $this->dataFile )
            );

            if ( $dataFileExists ) {
                $dataExists       = true;
                $this->wpInstalls = $this->getDataFileData($this->dataFile);
            }

            if ( $this->customDataFile != '' ) {
                $custDataFileExists = DAEscalate::escalate(
                    true,
                    DAEscalate::MGR_CMD_DATA_FILE_EXISTS,
                    array( 'filePath' => $this->customDataFile )
                );

                if ( $custDataFileExists ) {
                    $dataExists           = true;
                    $this->custWpInstalls =
                        $this->getDataFileData($this->customDataFile);
                }
            }
        }
        catch ( DAPluginException $e ) {
            Logger::debug($e->getMessage());
            return $e->getCode();
        }
        catch ( LSCMException $e ) {
            Logger::debug($e->getMessage());
            return $e->getCode();
        }

        if ( !$dataExists ) {
            return self::ERR_NOT_EXIST;
        }

        return 0;
    }

    /**
     *
     * @since 2.3
     *
     * @param string $dataFile
     *
     * @return false|string
     *
     * @throws LSCMException  Rethrown when DAEscalate::escalate() call throws a
     *     DAPluginException exception.
     */
    protected static function getDataFileContents( $dataFile )
    {
        try {
            return DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_GET_DATA_FILE_CONTENT,
                array( 'filePath' => $dataFile )
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param string $dataFile
     *
     * @return DAPanelWPInstall[]
     *
     * @throws LSCMException Thrown indirectly by parent::getDataFileData()
     *     call.
     */
    protected function getDataFileData( $dataFile )
    {
        $wpInstalls = parent::getDataFileData($dataFile);

        $daPanelWpInstalls = array();

        foreach ( $wpInstalls as $wpInstall ) {
            $wpInstallPath    = $wpInstall->getPath();
            $daPanelWpInstall = new DAPanelWPInstall($wpInstallPath);

            $daPanelWpInstall->initData($wpInstall->getData());
            $daPanelWpInstalls[$wpInstallPath] = $daPanelWpInstall;
        }

        unset($wpInstalls);

        return $daPanelWpInstalls;
    }

    /**
     *
     * @since 2.1
     *
     * @param string $path
     *
     * @return WPInstall|null
     *
     * @throws LSCMException  Re-thrown when DAEscalate::escalate() call throws
     *     a DAPluginException exception.
     */
    public function getWPInstall( $path )
    {
        try {
            $realPath = DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_GET_REAL_PATH,
                array( 'filePath' => $path )
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }

        $index = ($realPath !== false) ? $realPath : $path;

        if ( isset($this->wpInstalls[$index]) ) {
            return $this->wpInstalls[$index];
        }

        if ( isset($this->custWpInstalls[$index]) ) {
            return $this->custWpInstalls[$index];
        }

        return null;
    }

    /**
     *
     * @since 2.0
     *
     * @param string $dataFile
     * @param string $dataFileVer
     *
     * @return bool
     *
     * @throws LSCMException  Re-thrown when DAEscalate::escalate() call throws
     *     a DAPluginException exception.
     */
    public static function updateDataFile( $dataFile, $dataFileVer )
    {
        try {
            return DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_UPDATE_DATA_FILE,
                array( 'dataFile' => $dataFile, 'dataFileVer' => $dataFileVer )
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param string      $action
     * @param null|string $list
     * @param string[]    $extraArgs
     *
     * @return string[]
     *
     * @throws LSCMException  Re-thrown when DAEscalate::escalate() call throws
     *     a DAPluginException exception.
     * @throws LSCMException  Thrown indirectly by $this->init() call.
     * @throws LSCMException  Thrown indirectly by Logger::uiError() call.
     */
    public function doAction( $action, $list, array $extraArgs = array() )
    {
        try {
            $ret = DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_DO_ACTION,
                array(
                    'action'    => $action,
                    'list'      => $list,
                    'extraArgs' => $extraArgs
                )
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }

        /**
         * Re-populate $this->wpInstalls
         */
        $this->error = $this->init();

        $workingQueue    = $ret['workingQueue'];
        $invalidPathErrs = $ret['invalidPathErrs'];

        if ( !empty($workingQueue) ) {
            $this->workingQueue =
                array_merge($this->workingQueue, $workingQueue);
        }

        if ( !empty($invalidPathErrs) ) {

            foreach( $invalidPathErrs as $errMsg ) {
                Logger::uiError($errMsg);
            }
        }

        if ( $action == parent::CMD_SCAN || $action == parent::CMD_SCAN2 ) {
            /**
             * Explicitly clear any data file errors after scanning in case of
             * multiple actions performed in the same process (cli).
             */
            $this->error = 0;
        }

        return $ret['finishedList'];
    }

}
