<?php

/** ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019-2020
 * @since 2.0
 * ******************************************* */

namespace LsPanel\Extend;

use \Lsc\Wp\LSCMException;
use \Lsc\Wp\Panel\DirectAdmin;
use \LsPanel\DAEscalate;
use \LsPanel\DALiteSpeedUtil;
use \LsPanel\DAPluginException;

/**
 * @since 2.0
 */
class DirectAdminPanel extends DirectAdmin
{

    /**
     *
     * @since 2.0
     */
    public function __construct()
    {
        /**
         * Exception will not be thrown for DirectAdmin class.
         *
         * @noinspection PhpUnhandledExceptionInspection
         */
        parent::__construct();
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     * @throws LSCMException  Thrown directly and indirectly.
     */
    protected function serverCacheRootSearch()
    {
        try {
            $apacheConfExists = DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_FILE_EXISTS,
                array('filePath' => $this->apacheConf)
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }

        if ( $apacheConfExists ) {
            return $this->getCacheRootSetting($this->apacheConf);
        }

        return '';
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $svrCacheRoot
     * @throws LSCMException
     */
    public function setServerCacheRoot( $svrCacheRoot = '' )
    {
        try {
            DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_SET_SERVER_CACHE_ROOT
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }

        $this->serverCacheRoot = $this->defaultSvrCacheRoot;
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $vhConf
     * @param string  $vhCacheRoot
     * @throws LSCMException
     */
    public function writeVHCacheRoot( $vhConf, $vhCacheRoot = 'lscache' )
    {
        try {
            DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_WRITE_VH_CACHE_ROOT,
                array('vhConf' => $vhConf)
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $confDir  Directory to be searched.
     * @return string
     * @throws LSCMException
     */
    public function daVhCacheRootSearch( $confDir )
    {
        try {
            $cacheRoot = DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_DA_VH_CACHE_ROOT_SEARCH,
                array('confDir' => $confDir)
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }

        return $cacheRoot;
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $file
     * @return string
     * @throws LSCMException
     */
    public function getCacheRootSetting( $file )
    {
        try {
            $cacheRoot = DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_GET_CACHE_ROOT_SETTING,
                array( 'confFile' => $file )
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }

        return $cacheRoot;
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     * @throws LSCMException
     */
    public function getLSWSCacheRootSetting()
    {
        try {
            $svrCacheRoot = DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_GET_LSWS_CACHE_ROOT_SETTING
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }

        return $svrCacheRoot;
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $vhConf
     * @param string  $vhCacheRoot
     * @throws LSCMException
     */
    public function createVHConfAndSetCacheRoot( $vhConf,
            $vhCacheRoot = 'lscache' )
    {
        try {
            DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_CREATE_VHCONF_AND_SET_CACHE_ROOT,
                array( 'vhConf' => $vhConf )
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }
    }

    /**
     *
     * @since 2.0
     *
     * @throws LSCMException
     */
    public function applyVHConfChanges()
    {
        try {
            DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_APPLY_VHCONF_CHANGES
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }
    }

    /**
     *
     * @since 2.0
     *
     * @throws LSCMException
     */
    protected function prepareDocrootMap()
    {
        try {
            $this->docRootMap =
                DAEscalate::escalate(true, DAEscalate::MGR_CMD_GET_DOCROOT_MAP);
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }
    }

    /**
     *
     * @since 2.0
     *
     * @throws LSCMException  Thrown directly and indirectly.
     */
    public function verifyCacheSetup()
    {
        if ( !$this->isCacheEnabled() ) {
            throw new LSCMException(
                'LSCache is not included in the current LiteSpeed license. '
                    . 'Please purchase the LSCache add-on or upgrade to a '
                    . 'license type that includes LSCache and try again.',
                LSCMException::E_PERMISSION
            );
        }

        $restartRequired = false;

        if ( self::NOT_SET == $this->getServerCacheRoot() ) {
            $this->setServerCacheRoot();
            $restartRequired = true;
        }

        if ( self::NOT_SET == $this->getVHCacheRoot() ) {
            $this->setVHCacheRoot();
            $restartRequired = true;
        }

        if ( $restartRequired && DALiteSpeedUtil::getLSPID() != 0 ) {

            try {
                DALiteSpeedUtil::restartLSWS();
            }
            catch ( DAPluginException $e ) {
                throw new LSCMException($e->getMessage(), $e->getCode());
            }
        }
    }

    /**
     *
     * @since 2.0
     *
     * @return boolean
     * @throws LSCMException
     */
    public function isCacheEnabled()
    {
        try {
            return DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_IS_CACHE_ENABLED
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }
    }

    /**
     *
     * @since 2.1
     *
     * @throws LSCMException
     */
    public static function populateMinAPIVerFile()
    {
        try {
            DAEscalate::escalate(
                true,
                DAEscalate::MGR_CMD_POPULATE_MIN_API_VER_FILE
            );
        }
        catch ( DAPluginException $e ) {
            throw new LSCMException($e->getMessage(), $e->getCode());
        }
    }

}
