<?php

/** ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019-2020
 * @since 2.0
 * ******************************************* */

namespace LsPanel\View\Model;

use \Lsc\Wp\Context\Context;
use \Lsc\Wp\LSCMException;
use \Lsc\Wp\Panel\ControlPanel;
use \LsPanel\DAPluginException;
use \LsPanel\DAPluginLogger;
use \LsPanel\Extend\DirectAdminPanel;

/**
 * @since 2.0
 */
class CacheRootSetupViewModel extends BaseViewModel
{

    /**
     * @since 2.0
     * @var string
     */
    const FLD_ICON = 'icon';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_SVR_CACHE_ROOT = 'svr_cache_root';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_VH_CACHE_ROOT = 'vh_cache_root';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_MISSING = 'missing';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_ERR_MSGS = 'errMsgs';

    /**
     * @since 2.0
     * @var DirectAdminPanel
     */
    private $panelEnv;

    /**
     *
     * @since 2.0
     *
     * @param DirectAdminPanel  $panelEnv
     * @throws DAPluginException  Thrown indirectly.
     */
    public function __construct( DirectAdminPanel $panelEnv )
    {
        parent::__construct();

        $this->panelEnv = $panelEnv;

        $this->init();
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown indirectly.
     */
    private function init()
    {
        $this->setIconPath();
        $this->setCacheRootData();
        $this->setErrMsgsData();
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown indirectly.
     */
    private function setIconPath()
    {
        $iconPath = '';

        try
        {
            $iconDir = Context::getOption()->getIconDir();
            $iconPath = "{$iconDir}/cacheRootSetup.svg";
        }
        catch ( LSCMException $e )
        {
            DAPluginLogger::debug(
                $e->getMessage() . ' Could not get icon directory.'
            );
        }

        $this->tplData[self::FLD_ICON] = $iconPath;
    }

    /**
     *
     * @since 2.0
     */
    private function setCacheRootData()
    {
        $missing = false;
        $svrCacheRoot = $this->panelEnv->getServerCacheRoot();
        $vhCacheRoot = $this->panelEnv->getVHCacheRoot();

        if ( $svrCacheRoot != ControlPanel::NOT_SET ) {
            $svr = '  ' . htmlspecialchars(rtrim($svrCacheRoot, '/')) . '/  ';
        }
        else {
            $svr = 'not set!';
            $missing = true;
        }

        $this->tplData[self::FLD_SVR_CACHE_ROOT] = $svr;

        if ( $vhCacheRoot != ControlPanel::NOT_SET ) {
            $vh = '  ' . htmlspecialchars(rtrim($vhCacheRoot, '/'));

            if ( $vhCacheRoot[0] != '/' ) {
                $vh .= '<b>*</b> ';
            }
        }
        else {
            $vh = 'not set!';
            $missing = true;
        }

        $this->tplData[self::FLD_VH_CACHE_ROOT] = $vh;

        $this->tplData[self::FLD_MISSING] = $missing;
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown indirectly.
     */
    private function setErrMsgsData()
    {
        $errMsgs = DAPluginLogger::getUiMsgs(DAPluginLogger::UI_ERR);

        $this->tplData[self::FLD_ERR_MSGS] = $errMsgs;
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     */
    public function getTpl()
    {
        return realpath(__DIR__ . '/../Tpl') . '/CacheRootSetup.tpl';
    }

}
