<?php

/** ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author    Michael Alegre
 * @copyright 2019-2023 LiteSpeed Technologies, Inc.
 * @since     2.0
 * ******************************************* */

namespace LsPanel\View\Model;

use Lsc\Wp\LSCMException;
use LsPanel\DALiteSpeedControllerBase;
use LsPanel\DALiteSpeedUtil;
use LsPanel\DAPluginException;
use LsPanel\DAPluginLogger;
use LsPanel\Extend\DAPanelPluginVersion;

/**
 * @since 2.0
 */
class MainMenuViewModel extends BaseViewModel
{

    /**
     * @since 2.0
     * @var string
     */
    const FLD_AP_PID = 'apPid';

    /**
     * @since 2.3
     * @var string
     */
    const FLD_LITESPEED_INSTALLED = 'litespeedInstalled';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_LS_PID = 'lsPid';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_MAIN_MENU_INFO = 'mainMenuInfo';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_PLUGIN_VER = 'pluginVer';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_CURR_LSCWP_VER = 'currLscwpVer';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_LATEST_LSCWP_VER = 'newLscwpVer';

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown indirectly by $this->init() call.
     */
    public function __construct()
    {
        parent::__construct();

        $this->init();
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown indirectly by $this->setPidData()
     *     call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->setMainMenuInfoData() call.
     * @throws DAPluginException  Thrown indirectly by
     *     $this->setLscwpVerData() call.
     */
    protected function init( )
    {
        $this->setPidData();
        $this->setDAPluginVerData();

        if ( $this->setLitespeedInstalled() ) {
            $this->setMainMenuInfoData();
            $this->setLscwpVerData();
        }
    }

    /**
     *
     * @since 2.3
     *
     * @return bool
     */
    protected function setLitespeedInstalled()
    {
        if ( DALiteSpeedUtil::getLswsHome() != '' ) {
            return ($this->tplData[self::FLD_LITESPEED_INSTALLED] = true);
        }

        return ($this->tplData[self::FLD_LITESPEED_INSTALLED] = false);
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown when a running LiteSpeed process is
     *     detected while LSWS_HOME is undefined.
     */
    protected function setPidData()
    {
        $lsPid = DALiteSpeedUtil::getLSPID();

        if ( $lsPid > 0 && DALiteSpeedUtil::getLswsHome() == '' ) {
            throw new DAPluginException(
                'lsPid detected with LSWS_HOME undefined'
            );
        }

        $this->tplData[self::FLD_AP_PID] = DALiteSpeedUtil::getApachePID();
        $this->tplData[self::FLD_LS_PID] = $lsPid;
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Thrown indirectly by
     *     DALiteSpeedUtil::getMainMenuInfo() call.
     */
    protected function setMainMenuInfoData()
    {
        $this->tplData[self::FLD_MAIN_MENU_INFO] =
            DALiteSpeedUtil::getMainMenuInfo();
    }

    /**
     *
     * @since 2.0
     */
    protected function setDAPluginVerData()
    {
        $this->tplData[self::FLD_PLUGIN_VER] =
            DALiteSpeedControllerBase::MODULE_VERSION;
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException  Re-thrown when
     *     DAPanelPluginVersion::getInstance() call throws an LSCMException
     *     exception.
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::debug()
     *     call.
     * @throws DAPluginException  Thrown indirectly by DAPluginLogger::debug()
     *     call.
     */
    protected function setLscwpVerData()
    {
        $currLscwpVer = $latestLscwpVer = '';

        if ( DALiteSpeedUtil::sharedCodeLoaded() ) {

            try {
                /* @var DAPanelPluginVersion $verMgr */
                $verMgr = DAPanelPluginVersion::getInstance();
            }
            catch ( LSCMException $e ) {
                throw new DAPluginException($e->getMessage(), $e->getCode());
            }

            try {
                $currLscwpVer = $verMgr->getCurrentVersion();
            }
            catch ( LSCMException $e ) {
                DAPluginLogger::debug(
                    $e->getMessage() . ' Could not get active LSCWP version.'
                );
            }

            try {
                $latestLscwpVer = $verMgr->getLatestVersion();
            }
            catch ( LSCMException $e ) {
                DAPluginLogger::debug(
                    $e->getMessage() . ' Could not get latest LSCWP version.'
                );
            }
        }

        $this->tplData[self::FLD_CURR_LSCWP_VER] = $currLscwpVer;
        $this->tplData[self::FLD_LATEST_LSCWP_VER] = $latestLscwpVer;
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     */
    public function getTpl()
    {
        return realpath(__DIR__ . '/../Tpl') . '/MainMenu.tpl';
    }

}
