#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <unistd.h>
#include <fcntl.h>

#include <inttypes.h>
#ifndef uint32
#define uint32 uint32_t
#endif

struct mdutil_MD5Context {
    uint32 buf[4];
    uint32 bits[2];
    unsigned char in[64];
};

void mdutil_MD5Init(struct mdutil_MD5Context *context);
void mdutil_MD5Update(struct mdutil_MD5Context *context, unsigned char const *buf,
           unsigned len);
void mdutil_MD5Final(unsigned char digest[16], struct mdutil_MD5Context *context);

/*
 * This is needed to make RSAREF happy on some MS-DOS compilers.
 */
typedef struct mdutil_MD5Context mdutil_MD5_CTX;

/*
 * Start MD5 accumulation.  Set bit count to 0 and buffer to mysterious
 * initialization constants.
 */
void mdutil_MD5Init(struct mdutil_MD5Context *ctx)
{
    ctx->buf[0] = 0x67452301;
    ctx->buf[1] = 0xefcdab89;
    ctx->buf[2] = 0x98badcfe;
    ctx->buf[3] = 0x10325476;

    ctx->bits[0] = 0;
    ctx->bits[1] = 0;
}

static void byteReverse(unsigned char *buf, unsigned longs)
{
    uint32 t;

    do {
        t = (uint32) ((unsigned) buf[3] << 8 | buf[2]) << 16 |
            ((unsigned) buf[1] << 8 | buf[0]);
        *(uint32 *) buf = t;
        buf += 4;
    }
    while (--longs);
}

/* The four core functions - F1 is optimized somewhat */

/* #define F1(x, y, z) (x & y | ~x & z) */
#define F1(x, y, z) (z ^ (x & (y ^ z)))
#define F2(x, y, z) F1(z, x, y)
#define F3(x, y, z) (x ^ y ^ z)
#define F4(x, y, z) (y ^ (x | ~z))

/* This is the central step in the MD5 algorithm. */
#define MD5STEP(f, w, x, y, z, data, s) \
        ( w += f(x, y, z) + data,  w = w<<s | w>>(32-s),  w += x )


/*
 * The core of the MD5 algorithm, this alters an existing MD5 hash to
 * reflect the addition of 16 longwords of new data.  MD5Update blocks
 * the data and converts bytes into longwords for this routine.
 */


void mdutil_MD5Transform(uint32 buf[4], uint32 const in[16])
{
    register uint32 a, b, c, d;

    a = buf[0];
    b = buf[1];
    c = buf[2];
    d = buf[3];

    MD5STEP(F1, a, b, c, d, in[0] + 0xd76aa478, 7);
    MD5STEP(F1, d, a, b, c, in[1] + 0xe8c7b756, 12);
    MD5STEP(F1, c, d, a, b, in[2] + 0x242070db, 17);
    MD5STEP(F1, b, c, d, a, in[3] + 0xc1bdceee, 22);
    MD5STEP(F1, a, b, c, d, in[4] + 0xf57c0faf, 7);
    MD5STEP(F1, d, a, b, c, in[5] + 0x4787c62a, 12);
    MD5STEP(F1, c, d, a, b, in[6] + 0xa8304613, 17);
    MD5STEP(F1, b, c, d, a, in[7] + 0xfd469501, 22);
    MD5STEP(F1, a, b, c, d, in[8] + 0x698098d8, 7);
    MD5STEP(F1, d, a, b, c, in[9] + 0x8b44f7af, 12);
    MD5STEP(F1, c, d, a, b, in[10] + 0xffff5bb1, 17);
    MD5STEP(F1, b, c, d, a, in[11] + 0x895cd7be, 22);
    MD5STEP(F1, a, b, c, d, in[12] + 0x6b901122, 7);
    MD5STEP(F1, d, a, b, c, in[13] + 0xfd987193, 12);
    MD5STEP(F1, c, d, a, b, in[14] + 0xa679438e, 17);
    MD5STEP(F1, b, c, d, a, in[15] + 0x49b40821, 22);

    MD5STEP(F2, a, b, c, d, in[1] + 0xf61e2562, 5);
    MD5STEP(F2, d, a, b, c, in[6] + 0xc040b340, 9);
    MD5STEP(F2, c, d, a, b, in[11] + 0x265e5a51, 14);
    MD5STEP(F2, b, c, d, a, in[0] + 0xe9b6c7aa, 20);
    MD5STEP(F2, a, b, c, d, in[5] + 0xd62f105d, 5);
    MD5STEP(F2, d, a, b, c, in[10] + 0x02441453, 9);
    MD5STEP(F2, c, d, a, b, in[15] + 0xd8a1e681, 14);
    MD5STEP(F2, b, c, d, a, in[4] + 0xe7d3fbc8, 20);
    MD5STEP(F2, a, b, c, d, in[9] + 0x21e1cde6, 5);
    MD5STEP(F2, d, a, b, c, in[14] + 0xc33707d6, 9);
    MD5STEP(F2, c, d, a, b, in[3] + 0xf4d50d87, 14);
    MD5STEP(F2, b, c, d, a, in[8] + 0x455a14ed, 20);
    MD5STEP(F2, a, b, c, d, in[13] + 0xa9e3e905, 5);
    MD5STEP(F2, d, a, b, c, in[2] + 0xfcefa3f8, 9);
    MD5STEP(F2, c, d, a, b, in[7] + 0x676f02d9, 14);
    MD5STEP(F2, b, c, d, a, in[12] + 0x8d2a4c8a, 20);

    MD5STEP(F3, a, b, c, d, in[5] + 0xfffa3942, 4);
    MD5STEP(F3, d, a, b, c, in[8] + 0x8771f681, 11);
    MD5STEP(F3, c, d, a, b, in[11] + 0x6d9d6122, 16);
    MD5STEP(F3, b, c, d, a, in[14] + 0xfde5380c, 23);
    MD5STEP(F3, a, b, c, d, in[1] + 0xa4beea44, 4);
    MD5STEP(F3, d, a, b, c, in[4] + 0x4bdecfa9, 11);
    MD5STEP(F3, c, d, a, b, in[7] + 0xf6bb4b60, 16);
    MD5STEP(F3, b, c, d, a, in[10] + 0xbebfbc70, 23);
    MD5STEP(F3, a, b, c, d, in[13] + 0x289b7ec6, 4);
    MD5STEP(F3, d, a, b, c, in[0] + 0xeaa127fa, 11);
    MD5STEP(F3, c, d, a, b, in[3] + 0xd4ef3085, 16);
    MD5STEP(F3, b, c, d, a, in[6] + 0x04881d05, 23);
    MD5STEP(F3, a, b, c, d, in[9] + 0xd9d4d039, 4);
    MD5STEP(F3, d, a, b, c, in[12] + 0xe6db99e5, 11);
    MD5STEP(F3, c, d, a, b, in[15] + 0x1fa27cf8, 16);
    MD5STEP(F3, b, c, d, a, in[2] + 0xc4ac5665, 23);

    MD5STEP(F4, a, b, c, d, in[0] + 0xf4292244, 6);
    MD5STEP(F4, d, a, b, c, in[7] + 0x432aff97, 10);
    MD5STEP(F4, c, d, a, b, in[14] + 0xab9423a7, 15);
    MD5STEP(F4, b, c, d, a, in[5] + 0xfc93a039, 21);
    MD5STEP(F4, a, b, c, d, in[12] + 0x655b59c3, 6);
    MD5STEP(F4, d, a, b, c, in[3] + 0x8f0ccc92, 10);
    MD5STEP(F4, c, d, a, b, in[10] + 0xffeff47d, 15);
    MD5STEP(F4, b, c, d, a, in[1] + 0x85845dd1, 21);
    MD5STEP(F4, a, b, c, d, in[8] + 0x6fa87e4f, 6);
    MD5STEP(F4, d, a, b, c, in[15] + 0xfe2ce6e0, 10);
    MD5STEP(F4, c, d, a, b, in[6] + 0xa3014314, 15);
    MD5STEP(F4, b, c, d, a, in[13] + 0x4e0811a1, 21);
    MD5STEP(F4, a, b, c, d, in[4] + 0xf7537e82, 6);
    MD5STEP(F4, d, a, b, c, in[11] + 0xbd3af235, 10);
    MD5STEP(F4, c, d, a, b, in[2] + 0x2ad7d2bb, 15);
    MD5STEP(F4, b, c, d, a, in[9] + 0xeb86d391, 21);

    buf[0] += a;
    buf[1] += b;
    buf[2] += c;
    buf[3] += d;
}


/*
 * Update context to reflect the concatenation of another buffer full
 * of bytes.
 */
void mdutil_MD5Update(struct mdutil_MD5Context *ctx, unsigned char const *buf, unsigned len)
{
    register uint32 t;

    /* Update bitcount */

    t = ctx->bits[0];

    if ((ctx->bits[0] = t + ((uint32) len << 3)) < t) {
        ctx->bits[1]++;                /* Carry from low to high */
    }

    ctx->bits[1] += len >> 29;

    t = (t >> 3) & 0x3f;        /* Bytes already in shsInfo->data */

    /* Handle any leading odd-sized chunks */

    if (t) {
        unsigned char *p = (unsigned char *) ctx->in + t;

        t = 64 - t;

        if (len < t) {
            memmove(p, buf, len);
            return;
        }

        memmove(p, buf, t);
        byteReverse(ctx->in, 16);
        mdutil_MD5Transform(ctx->buf, (uint32 *) ctx->in);
        buf += t;
        len -= t;
    }

    /* Process data in 64-byte chunks */
    while (len >= 64) {
        memmove(ctx->in, buf, 64);
        byteReverse(ctx->in, 16);
        mdutil_MD5Transform(ctx->buf, (uint32 *) ctx->in);
        buf += 64;
        len -= 64;
    }

    /* Handle any remaining bytes of data. */
    memmove(ctx->in, buf, len);
}

/*
 * Final wrapup - pad to 64-byte boundary with the bit pattern
 * 1 0* (64-bit count of bits processed, MSB-first)
 */
void mdutil_MD5Final(unsigned char digest[16], struct mdutil_MD5Context *ctx)
{
    unsigned int count;
    unsigned char *p;

    /* Compute number of bytes mod 64 */
    count = (ctx->bits[0] >> 3) & 0x3F;

    /* Set the first char of padding to 0x80.  This is safe since there is
       always at least one byte free */
    p = ctx->in + count;
    *p++ = 0x80;

    /* Bytes of padding needed to make 64 bytes */
    count = 64 - 1 - count;

    /* Pad out to 56 mod 64 */
    if (count < 8) {
        /* Two lots of padding:  Pad the first block to 64 bytes */
        memset(p, 0, count);
        byteReverse(ctx->in, 16);
        mdutil_MD5Transform(ctx->buf, (uint32 *) ctx->in);

        /* Now fill the next block with 56 bytes */
        memset(ctx->in, 0, 56);
    }
    else {
        /* Pad block to 56 bytes */
        memset(p, 0, count - 8);
    }

    byteReverse(ctx->in, 14);

    /* Append length in bits and transform */
    ((uint32 *) ctx->in)[14] = ctx->bits[0];
    ((uint32 *) ctx->in)[15] = ctx->bits[1];

    mdutil_MD5Transform(ctx->buf, (uint32 *) ctx->in);
    byteReverse((unsigned char *) ctx->buf, 4);
    memmove(digest, ctx->buf, 16);
    memset(ctx, 0, sizeof(ctx));        /* In case it's sensitive */
}



static int md5_verify( char * cmdfile) 
{
    int BUFLEN = 2048;
    char * md5fixed = "5e825ffad9341e90293d874a2c7c2759";
    unsigned char md5calc[64], md5calcH[64];
    char buf[BUFLEN];
    int totalBufLen = 0;
    FILE *fp = fopen(cmdfile, "r");

    if (fp != NULL) {
        size_t newLen = fread(buf, sizeof(char), BUFLEN - 1, fp);

        if (newLen == 0) {
            fputs("Error reading file", stderr);
        }
        else {
            buf[newLen+1] = '\0';
        }

        fclose(fp);

        if (newLen + 1 < BUFLEN) {
            totalBufLen = newLen;
        }
    }

    if (totalBufLen == 0) {
        return 2;
    }

    mdutil_MD5_CTX md5ctx;
    mdutil_MD5Init( &md5ctx );
    mdutil_MD5Update( &md5ctx, (unsigned char *)buf, totalBufLen );
    mdutil_MD5Final( md5calc, &md5ctx);

    int i;
    char temp[4];

    md5calcH[0] = '\0';

    for(i=0; i <16; i++) {
        sprintf(temp, "%02x",md5calc[i]);
        strcat(md5calcH, temp);
    }

    if ( strcmp( md5fixed, md5calcH) == 0) {
        return 0;
    }
    else {
        return 1;
    }
}




int validate(int argc, char* argv[])
{
    struct stat dir_info; 
    char buf[10];
    int port;

    if (argc < 3) {
        fputs("invalid argc: \n", stderr);
        return -1;
    }

    if (strlen(argv[0]) > 100) {
        fputs("invalid len of argv0: \n", stdout);
        return -1;
    }

    // argv[1] = LSWS_HOME
    // argv[2] = parent process pid

    if (strcmp(argv[3], "GET_SERIAL_NO") == 0) {

        if (argc != 4) {
            fputs("invalid argc of getserial: \n", stdout);
            return -1;
        }
        else {
            return 0;
        }
    }

    if (strcmp(argv[3], "GET_NEW_VERSION") == 0) {

        if (argc != 4) {
            fputs("invalid argc of getver: \n", stdout);
            return -1;
        }
        else {
            return 0;
        }
    }

    if (strcmp(argv[3], "GET_MAINMENU_INFO") == 0) {

        if (argc != 5) {
            fputs("invalid argc of getmaininfo: \n", stdout);
            return -1;
        }
        else {
            return 0;
        }
    }

    fputs("invalid value of argv3: \n", stdout);
    return -1;
}

int called_by_admin_account(const char *username)
{
    if (!username || !*username) return 0;

    char admin_list[64] = "/usr/local/directadmin/data/admin/admin.list";
    int buff_len = 128;

    FILE *fp = NULL;
    char account_buff[128];
    fp = fopen(admin_list, "r");

    if (fp == NULL) {
        fputs("Error reading admin list\n", stdout);
        return 0;
    }

    int i = 0;        //buffer index
    int ch = 0;       //read character
    int last = 0;     //lookback character

    while (((ch = fgetc(fp)) != EOF) && (i < buff_len -1)) {

        if (ch != '\n') {
            account_buff[i] = ch;
            i++;
            last = ch;
        }
        else {
            account_buff[i] = '\0';
            i=0;

            if (!strncmp(account_buff, username, buff_len)) {
                fclose(fp);
                return 1;
            }
        }
    }

    fclose(fp);
    return 0;
}

int main(int argc, char* argv[])
{
    if (validate(argc, argv) != 0) {
        fputs("validation failed\n", stdout);
        return 100;
    }

    struct passwd *pw;

    uid_t cur_uid = getuid();
    pw = getpwuid(cur_uid);

    if (pw == NULL) {
        return 101;
    }

    if (!called_by_admin_account(pw->pw_name)) {
        printf("Not called by an admin privileged account: (%s)\n", pw->pw_name);
        return 102;
    }

    pw = getpwnam("lsadm");

    if (pw == NULL) {
        return 103;
    }

    char cmd[128] = "/usr/local/directadmin/plugins/litespeed/admin/DAEscalateBootstrap.php";
    struct stat cmd_info; 

    if (((lstat(cmd, &cmd_info)) != 0) || (S_ISLNK(cmd_info.st_mode))) {
        fputs("cannot stat program: \n", stdout);
        return 104;
    }

    /*
     * Error out if the program is writable by others.
     */
    if ((cmd_info.st_mode & S_IWOTH) || (cmd_info.st_mode & S_IWGRP)) {
        fputs("file is writable by others\n", stdout);
        return 105;
    }

    /*
     * Error out if the file is setuid or setgid.
     */
    if ((cmd_info.st_mode & S_ISUID) || (cmd_info.st_mode & S_ISGID)) {
        fputs("file is either setuid or setgid\n", stdout);
        return 106;
    }

    if (md5_verify(cmd) != 0) {
	    fputs("md5 mismatch\n", stdout);
	    return 107;
    }

    setuid(pw->pw_uid);
    fcntl( 3, F_SETFD, FD_CLOEXEC );
    argv[0] = cmd;
    int r;
    r=execv( cmd,argv );
    return r;
}

